# -*- coding: utf-8 -*-
# filename: test_async_integration_vrl_e2e.py
# @Time    : 2025/10/06 14:55
# @Author  : JQQ
# @Email   : jqq1716@gmail.com
# @Software: PyCharm
"""
中文: Computer-Agent-Server 三端联动的 VRL 功能端到端测试。
      验证 VRL 转换功能在完整工作流中的正确性：
      - Computer 配置 VRL 脚本
      - Agent 调用工具
      - Agent 收到的工具返回中包含 VRL 转换结果
English: Asynchronous end-to-end tests for VRL functionality in Computer-Agent-Server integration.
         Validates VRL transformation in complete workflow:
         - Computer configures VRL script
         - Agent calls tools
         - Agent receives tool results with VRL transformation
"""

from __future__ import annotations

import asyncio
import json
import sys
import time
from pathlib import Path
from typing import Any

import pytest

from a2c_smcp.agent import AsyncSMCPAgentClient, DefaultAgentAuthProvider
from a2c_smcp.agent.types import AsyncAgentEventHandler
from a2c_smcp.computer import Computer
from a2c_smcp.computer.mcp_clients.model import A2C_VRL_TRANSFORMED, StdioServerConfig
from a2c_smcp.computer.socketio.client import SMCPComputerClient
from a2c_smcp.smcp import (
    JOIN_OFFICE_EVENT,
    SMCP_NAMESPACE,
    EnterOfficeNotification,
    LeaveOfficeNotification,
    SMCPTool,
    UpdateMCPConfigNotification,
)

pytestmark = pytest.mark.e2e


async def _wait_until(cond, timeout: float = 3.0, step: float = 0.01) -> bool:
    """
    中文: 简易异步等待辅助函数，直到条件满足或超时。
    English: Simple async wait helper until condition met or timeout.
    """
    end = asyncio.get_event_loop().time() + timeout
    while asyncio.get_event_loop().time() < end:
        if cond():
            return True
        await asyncio.sleep(step)
    return cond()


class MockAsyncEventHandler(AsyncAgentEventHandler):
    """
    中文: 测试用的异步事件处理器，记录所有事件
    English: Test async event handler that records all events
    """

    def __init__(self):
        self.enter_office_events: list[EnterOfficeNotification] = []
        self.leave_office_events: list[LeaveOfficeNotification] = []
        self.update_config_events: list[UpdateMCPConfigNotification] = []
        self.tools_received_events: list[tuple[str, list[SMCPTool]]] = []

    async def on_computer_enter_office(self, data: EnterOfficeNotification, sio: AsyncSMCPAgentClient) -> None:
        self.enter_office_events.append(data)

    async def on_computer_leave_office(self, data: LeaveOfficeNotification, sio: AsyncSMCPAgentClient) -> None:
        self.leave_office_events.append(data)

    async def on_computer_update_config(self, data: UpdateMCPConfigNotification, sio: AsyncSMCPAgentClient) -> None:
        self.update_config_events.append(data)

    async def on_tools_received(self, computer: str, tools: list[SMCPTool], sio: AsyncSMCPAgentClient) -> None:
        self.tools_received_events.append((computer, tools))


def _create_mcp_config_with_vrl(name: str, script_path: str, vrl_script: str | None = None) -> dict[str, Any]:
    """
    中文: 创建带 VRL 配置的 MCP Server 配置
    English: Create MCP Server config with VRL script
    """
    project_root = Path(__file__).parent.parent.parent
    absolute_script_path = (project_root / script_path).resolve()

    config = {
        "name": name,
        "type": "stdio",
        "disabled": False,
        "forbidden_tools": [],
        "tool_meta": {},
        "default_tool_meta": {"auto_apply": True},
        "server_parameters": {
            "command": sys.executable,
            "args": [str(absolute_script_path)],
            "env": None,
            "cwd": None,
            "encoding": "utf-8",
            "encoding_error_handler": "strict",
        },
    }

    # 添加 VRL 配置 / Add VRL config
    if vrl_script:
        config["vrl"] = vrl_script

    return config


@pytest.mark.asyncio
async def test_async_integration_agent_receives_vrl_transformed_result(
    async_integration_socketio_server,
    async_integration_server_port: int,
    tmp_path: Path,
):
    """
    中文:
      - 验证 Computer 配置了 VRL 脚本
      - 验证 Agent 调用工具后收到的结果包含 VRL 转换数据
      - 验证 VRL 转换结果存储在 meta[a2c_vrl_transformed] 中
      - 验证转换后的数据结构符合 VRL 脚本定义
    English:
      - Verify Computer is configured with VRL script
      - Verify Agent receives VRL transformed result after tool call
      - Verify VRL transformation is stored in meta[a2c_vrl_transformed]
      - Verify transformed data structure matches VRL script definition
    """
    agent_id = "async-vrl-integration-office-1"
    office_id = agent_id
    server_url = f"http://127.0.0.1:{async_integration_server_port}"

    # 配置 VRL 脚本：添加转换标记和额外字段
    # Configure VRL script: add transformation marker and extra fields
    vrl_script = """
.vrl_transformed = true
.transformation_timestamp = now()
.status = "processed"
.original_tool_name = .content[0].text
"""

    # 创建带 VRL 配置的 MCP Server
    # Create MCP Server config with VRL
    mcp_config = _create_mcp_config_with_vrl(
        "e2e-vrl-integration-server-1",
        "tests/integration_tests/computer/mcp_servers/resources_subscribe_stdio_server.py",
        vrl_script=vrl_script,
    )
    stdio_config = StdioServerConfig(**mcp_config)

    # 创建 Computer 实例
    # Create Computer instance
    computer = Computer(
        name="test",
        mcp_servers={stdio_config},
        auto_connect=True,
    )
    computer_client = SMCPComputerClient(computer=computer)

    # 创建 Agent 客户端
    # Create Agent client
    auth_provider = DefaultAgentAuthProvider(
        agent_id=agent_id,
        office_id=office_id,
    )
    event_handler = MockAsyncEventHandler()
    agent_client = AsyncSMCPAgentClient(
        auth_provider=auth_provider,
        event_handler=event_handler,
    )

    try:
        start_time = time.time()

        # 1. Agent 先连接并加入办公室
        # Agent connects and joins office first
        await agent_client.connect_to_server(server_url)
        await asyncio.sleep(0.2)

        ok, err = await agent_client.call(
            JOIN_OFFICE_EVENT,
            {"role": "agent", "name": "test-agent-vrl-1", "office_id": office_id},
            namespace=SMCP_NAMESPACE,
            timeout=5,
        )
        assert ok is True, f"Agent join office failed: {err}"
        await asyncio.sleep(0.3)

        # 2. Computer 启动并连接到 Server
        # Computer boots up and connects to Server
        await computer.boot_up()
        await computer_client.connect(
            server_url,
            socketio_path="/socket.io",
            namespaces=[SMCP_NAMESPACE],
            transports=["polling"],
        )
        await computer_client.join_office(office_id)

        # 3. 等待事件传播
        # Wait for event propagation
        await asyncio.sleep(1.0)

        # 4. 验证 Agent 收到工具列表
        # Verify Agent received tool list
        assert await _wait_until(lambda: len(event_handler.tools_received_events) >= 1, timeout=5)
        computer_name, tools = event_handler.tools_received_events[0]
        assert computer_name == "test", "回调工具列表时，使用的是computer name"

        # 5. Agent 调用工具
        # Agent calls tool
        result = await agent_client.emit_tool_call(
            computer=computer_name,
            tool_name="mark_a",
            params={},
            timeout=15,
        )

        # 6. 验证工具调用成功
        # Verify tool call succeeded
        assert result.isError is False, f"Tool call failed: {result}"
        assert len(result.content) >= 1
        assert "ok:mark_a" in result.content[0].text

        # 7. 验证 VRL 转换结果存在于 meta 中
        # Verify VRL transformation result exists in meta
        # 注意：由于Pydantic模型的序列化问题，meta可能在model_extra中
        # Note: Due to Pydantic model serialization, meta might be in model_extra
        result_dict = result.model_dump()
        print(f"[E2E VRL] Full result dict keys: {result_dict.keys()}")
        print(f"[E2E VRL] Result meta field: {result.meta}")

        # 尝试从多个位置获取meta数据
        # Try to get meta data from multiple locations
        meta_data = result.meta or result_dict.get("meta") or {}

        assert meta_data, f"Tool result meta is empty. Result dict: {result_dict}"
        assert A2C_VRL_TRANSFORMED in meta_data, (
            f"VRL transformation key '{A2C_VRL_TRANSFORMED}' not found in meta. Available keys: {list(meta_data.keys())}"
        )

        # 8. 解析 VRL 转换结果
        # Parse VRL transformation result
        vrl_result_json = meta_data[A2C_VRL_TRANSFORMED]
        assert isinstance(vrl_result_json, str), f"VRL result should be JSON string, got {type(vrl_result_json)}"

        vrl_result = json.loads(vrl_result_json)
        print(f"[E2E VRL] VRL transformation result: {vrl_result}")

        # 9. 验证 VRL 转换后的字段
        # Verify VRL transformed fields
        assert "vrl_transformed" in vrl_result, f"'vrl_transformed' field not found in VRL result: {vrl_result}"
        assert vrl_result["vrl_transformed"] is True, f"'vrl_transformed' should be True, got {vrl_result['vrl_transformed']}"

        assert "status" in vrl_result, f"'status' field not found in VRL result: {vrl_result}"
        assert vrl_result["status"] == "processed", f"'status' should be 'processed', got {vrl_result['status']}"

        assert "transformation_timestamp" in vrl_result, f"'transformation_timestamp' field not found in VRL result: {vrl_result}"

        assert "original_tool_name" in vrl_result, f"'original_tool_name' field not found in VRL result: {vrl_result}"
        assert "ok:mark_a" in vrl_result["original_tool_name"], (
            f"'original_tool_name' should contain 'ok:mark_a', got {vrl_result['original_tool_name']}"
        )

        print(f"[E2E VRL] Test completed successfully in {time.time() - start_time:.2f}s")

    finally:
        # 清理资源
        # Cleanup resources
        cleanup_start = time.time()

        await agent_client.disconnect()
        print(f"[E2E VRL] Agent disconnect took {time.time() - cleanup_start:.2f}s")

        await computer.shutdown()
        print(f"[E2E VRL] Computer shutdown took {time.time() - cleanup_start:.2f}s")


@pytest.mark.asyncio
async def test_async_integration_vrl_field_mapping_and_extraction(
    async_integration_socketio_server,
    async_integration_server_port: int,
    tmp_path: Path,
):
    """
    中文:
      - 验证 VRL 可以提取和重组工具返回的数据结构
      - 验证 Agent 可以获取经过 VRL 处理后的结构化数据
      - 验证复杂的 VRL 转换逻辑（字段提取、嵌套对象等）
    English:
      - Verify VRL can extract and reorganize tool return data structure
      - Verify Agent can receive VRL-processed structured data
      - Verify complex VRL transformation logic (field extraction, nested objects, etc.)
    """
    agent_id = "async-vrl-integration-office-2"
    office_id = agent_id
    server_url = f"http://127.0.0.1:{async_integration_server_port}"

    # 配置复杂的 VRL 脚本：提取内容并重组为新结构
    # Configure complex VRL script: extract content and reorganize into new structure
    vrl_script = """
.result = {
    "success": true,
    "data": {
        "tool_output": .content[0].text,
        "content_type": .content[0].type
    },
    "metadata": {
        "is_error": .isError,
        "processed_at": now(),
        "processor": "vrl-e2e-test"
    }
}
.summary = "Tool executed successfully"
"""

    # 创建带 VRL 配置的 MCP Server
    # Create MCP Server config with VRL
    mcp_config = _create_mcp_config_with_vrl(
        "e2e-vrl-integration-server-2",
        "tests/integration_tests/computer/mcp_servers/resources_subscribe_stdio_server.py",
        vrl_script=vrl_script,
    )
    stdio_config = StdioServerConfig(**mcp_config)

    # 创建 Computer 实例
    # Create Computer instance
    computer = Computer(
        name="test",
        mcp_servers={stdio_config},
        auto_connect=True,
    )
    computer_client = SMCPComputerClient(computer=computer)

    # 创建 Agent 客户端
    # Create Agent client
    auth_provider = DefaultAgentAuthProvider(
        agent_id=agent_id,
        office_id=office_id,
    )
    event_handler = MockAsyncEventHandler()
    agent_client = AsyncSMCPAgentClient(
        auth_provider=auth_provider,
        event_handler=event_handler,
    )

    try:
        # 1. Agent 先连接并加入办公室
        # Agent connects and joins office first
        await agent_client.connect_to_server(server_url)
        await asyncio.sleep(0.2)

        await agent_client.call(
            JOIN_OFFICE_EVENT,
            {"role": "agent", "name": "test-agent-vrl-2", "office_id": office_id},
            namespace=SMCP_NAMESPACE,
            timeout=5,
        )
        await asyncio.sleep(0.3)

        # 2. Computer 启动并连接到 Server
        # Computer boots up and connects to Server
        await computer.boot_up()
        await computer_client.connect(
            server_url,
            socketio_path="/socket.io",
            namespaces=[SMCP_NAMESPACE],
            transports=["polling"],
        )
        await computer_client.join_office(office_id)

        # 3. 等待事件传播
        # Wait for event propagation
        await asyncio.sleep(1.0)

        # 4. 验证 Agent 收到工具列表
        # Verify Agent received tool list
        assert await _wait_until(lambda: len(event_handler.tools_received_events) >= 1, timeout=5)
        computer_name, tools = event_handler.tools_received_events[0]
        assert computer_name == "test", "回调工具列表时使用的是computer name"

        # 5. Agent 调用工具
        # Agent calls tool
        result = await agent_client.emit_tool_call(
            computer=computer_name,
            tool_name="mark_a",
            params={},
            timeout=15,
        )

        # 6. 验证工具调用成功
        # Verify tool call succeeded
        assert result.isError is False, f"Tool call failed: {result}"

        # 7. 验证 VRL 转换结果
        # Verify VRL transformation result
        result_dict = result.model_dump()
        meta_data = result.meta or result_dict.get("meta") or {}

        assert meta_data, f"Tool result meta is empty. Result dict: {result_dict}"
        assert A2C_VRL_TRANSFORMED in meta_data

        vrl_result = json.loads(meta_data[A2C_VRL_TRANSFORMED])
        print(f"[E2E VRL] Complex VRL transformation result: {json.dumps(vrl_result, indent=2)}")

        # 8. 验证重组后的数据结构
        # Verify reorganized data structure
        assert "result" in vrl_result, f"'result' field not found in VRL result: {vrl_result}"
        assert "success" in vrl_result["result"], "'result.success' not found"
        assert vrl_result["result"]["success"] is True

        assert "data" in vrl_result["result"], "'result.data' not found"
        assert "tool_output" in vrl_result["result"]["data"], "'result.data.tool_output' not found"
        assert "ok:mark_a" in vrl_result["result"]["data"]["tool_output"]

        assert "metadata" in vrl_result["result"], "'result.metadata' not found"
        assert "is_error" in vrl_result["result"]["metadata"], "'result.metadata.is_error' not found"
        assert vrl_result["result"]["metadata"]["is_error"] is False

        assert "summary" in vrl_result, f"'summary' field not found in VRL result: {vrl_result}"
        assert "successfully" in vrl_result["summary"].lower()

        print("[E2E VRL] Complex VRL transformation test completed successfully")

    finally:
        # 清理资源
        # Cleanup resources
        await agent_client.disconnect()
        await computer.shutdown()


@pytest.mark.asyncio
async def test_async_integration_vrl_preserves_original_result(
    async_integration_socketio_server,
    async_integration_server_port: int,
    tmp_path: Path,
):
    """
    中文:
      - 验证 VRL 转换不影响原始工具返回内容
      - 验证原始 content 字段保持不变
      - 验证 VRL 转换结果仅存储在 meta 中
    English:
      - Verify VRL transformation doesn't affect original tool return content
      - Verify original content field remains unchanged
      - Verify VRL transformation result is only stored in meta
    """
    agent_id = "async-vrl-integration-office-3"
    office_id = agent_id
    server_url = f"http://127.0.0.1:{async_integration_server_port}"

    # 配置 VRL 脚本
    # Configure VRL script
    vrl_script = '.vrl_marker = "this_is_vrl_transformed_data"\n.extra_info = {"key": "value"}'

    # 创建带 VRL 配置的 MCP Server
    # Create MCP Server config with VRL
    mcp_config = _create_mcp_config_with_vrl(
        "e2e-vrl-integration-server-3",
        "tests/integration_tests/computer/mcp_servers/resources_subscribe_stdio_server.py",
        vrl_script=vrl_script,
    )
    stdio_config = StdioServerConfig(**mcp_config)

    # 创建 Computer 实例
    # Create Computer instance
    computer = Computer(
        name="test",
        mcp_servers={stdio_config},
        auto_connect=True,
    )
    computer_client = SMCPComputerClient(computer=computer)

    # 创建 Agent 客户端
    # Create Agent client
    auth_provider = DefaultAgentAuthProvider(
        agent_id=agent_id,
        office_id=office_id,
    )
    event_handler = MockAsyncEventHandler()
    agent_client = AsyncSMCPAgentClient(
        auth_provider=auth_provider,
        event_handler=event_handler,
    )

    try:
        # 1. Agent 先连接并加入办公室
        # Agent connects and joins office first
        await agent_client.connect_to_server(server_url)
        await asyncio.sleep(0.2)

        await agent_client.call(
            JOIN_OFFICE_EVENT,
            {"role": "agent", "name": "test-agent-vrl-3", "office_id": office_id},
            namespace=SMCP_NAMESPACE,
            timeout=5,
        )
        await asyncio.sleep(0.3)

        # 2. Computer 启动并连接到 Server
        # Computer boots up and connects to Server
        await computer.boot_up()
        await computer_client.connect(
            server_url,
            socketio_path="/socket.io",
            namespaces=[SMCP_NAMESPACE],
            transports=["polling"],
        )
        await computer_client.join_office(office_id)

        # 3. 等待事件传播
        # Wait for event propagation
        await asyncio.sleep(1.0)

        # 4. 验证 Agent 收到工具列表
        # Verify Agent received tool list
        assert await _wait_until(lambda: len(event_handler.tools_received_events) >= 1, timeout=5)
        computer_name, tools = event_handler.tools_received_events[0]
        assert computer_name == "test", "回调获取工具列表时使用的是 computer name"

        # 5. Agent 调用工具
        # Agent calls tool
        result = await agent_client.emit_tool_call(
            computer=computer_name,
            tool_name="mark_a",
            params={},
            timeout=15,
        )

        # 6. 验证工具调用成功
        # Verify tool call succeeded
        assert result.isError is False, f"Tool call failed: {result}"

        # 7. 验证原始内容保持不变
        # Verify original content remains unchanged
        assert len(result.content) >= 1, "Tool result content is empty"
        assert "ok:mark_a" in result.content[0].text, f"Original content should contain 'ok:mark_a', got {result.content[0].text}"

        # 8. 验证 VRL 转换结果存在于 meta 中，但不影响 content
        # Verify VRL transformation exists in meta but doesn't affect content
        result_dict = result.model_dump()
        meta_data = result.meta or result_dict.get("meta") or {}

        assert meta_data, f"Tool result meta is empty. Result dict: {result_dict}"
        assert A2C_VRL_TRANSFORMED in meta_data

        vrl_result = json.loads(meta_data[A2C_VRL_TRANSFORMED])
        print(f"[E2E VRL] VRL result in meta: {vrl_result}")

        # 9. 验证 VRL 转换的字段
        # Verify VRL transformed fields
        assert "vrl_marker" in vrl_result
        assert vrl_result["vrl_marker"] == "this_is_vrl_transformed_data"
        assert "extra_info" in vrl_result
        assert vrl_result["extra_info"]["key"] == "value"

        # 10. 确认原始 content 中不包含 VRL 转换的字段
        # Confirm original content doesn't contain VRL transformed fields
        content_text = result.content[0].text
        assert "vrl_marker" not in content_text, "VRL transformed field should not appear in original content"
        assert "this_is_vrl_transformed_data" not in content_text, "VRL transformed value should not appear in original content"

        print("[E2E VRL] VRL preservation test completed successfully")

    finally:
        # 清理资源
        # Cleanup resources
        await agent_client.disconnect()
        await computer.shutdown()



# -*- coding: utf-8 -*-
# filename: mock_uv_server.py
# @Time    : 2025/8/21 14:29
# @Author  : JQQ
# @Email   : jqq1716@gmail.com
# @Software: PyCharm

import asyncio

# 3rd party imports
import uvicorn

# socketio imports
from socketio import ASGIApp

PORT = 8000

# deactivate monitoring task in python-socketio to avoid errores during shutdown
# sio.eio.start_service_task = False

"""
参考：
https://github.com/miguelgrinberg/python-socketio/issues/332?utm_source=chatgpt.com
"""


class UvicornTestServer(uvicorn.Server):
    """Uvicorn test server

    Usage:
        @pytest.fixture
        async def start_stop_server():
            server = UvicornTestServer()
            await server.up()
            yield
            await server.down()
    """

    def __init__(self, app: ASGIApp = None, host: str = "127.0.0.1", port: int = PORT):
        """Create a Uvicorn test server

        Args:
            app (ASGIApp, optional): the ASGIApp app. Defaults to main.asgi_app.
            host (str, optional): the host ip. Defaults to '127.0.0.1'.
            port (int, optional): the port. Defaults to PORT.
        """
        self._startup_done = asyncio.Event()
        super().__init__(config=uvicorn.Config(app, host=host, port=port))

    async def startup(self, sockets: list | None = None) -> None:
        """Override uvicorn startup"""
        await super().startup(sockets=sockets)
        # self.config.setup_event_loop()  # 从0.36版本开始，不再需要这个方法
        self._startup_done.set()

    async def up(self) -> None:
        """Start up server asynchronously"""
        self._serve_task = asyncio.create_task(self.serve())
        await self._startup_done.wait()

    async def down(self, force: bool = False) -> None:
        """Shut down server asynchronously

        Args:
            force (bool): 中文: 是否强制快速关闭，跳过优雅关闭等待 / English: Force fast shutdown without graceful wait
        """
        self.should_exit = True
        if force:
            # 中文: 强制退出，不等待连接清理 / English: Force exit without waiting for connection cleanup
            self.force_exit = True
            # 中文: 取消服务任务以立即停止 / English: Cancel serve task to stop immediately
            if hasattr(self, "_serve_task") and not self._serve_task.done():
                self._serve_task.cancel()
                try:
                    # 中文: 减少超时时间到 0.2 秒以加快测试速度 / English: Reduce timeout to 0.2s to speed up tests
                    await asyncio.wait_for(self._serve_task, timeout=0.2)
                except (TimeoutError, asyncio.CancelledError):
                    pass
        else:
            await self._serve_task



"""
文件名: main.py
作者: JQQ
创建日期: 2025/9/18
最后修改日期: 2025/9/22
版权: 2023 JQQ. All rights reserved.
依赖: typer, rich, prompt_toolkit
描述:
  中文: A2C 计算机客户端的命令行入口，提供持续运行模式与基础交互命令。
  English: CLI entry for A2C Computer client. Provides persistent run mode and basic interactive commands.
"""

from __future__ import annotations

import asyncio
import json
from collections.abc import Callable
from pathlib import Path
from typing import Any

import typer
from prompt_toolkit import PromptSession
from prompt_toolkit.patch_stdout import patch_stdout
from pydantic import TypeAdapter

from a2c_smcp.computer.cli.interactive_impl import interactive_loop as _interactive_loop_impl
from a2c_smcp.computer.cli.utils import (
    parse_kv_pairs,
    resolve_import_target,
)
from a2c_smcp.computer.computer import Computer
from a2c_smcp.computer.inputs.resolver import InputResolver
from a2c_smcp.computer.mcp_clients.model import (
    MCPServerInput as MCPServerInputModel,
)
from a2c_smcp.computer.socketio.client import SMCPComputerClient
from a2c_smcp.computer.utils import console as console_util
from a2c_smcp.smcp import SMCP_NAMESPACE
from a2c_smcp.smcp import MCPServerConfig as SMCPServerConfigDict
from a2c_smcp.smcp import MCPServerInput as SMCPServerInputDict

app = typer.Typer(add_completion=False, help="A2C Computer CLI")
# 使用全局 Console（引用模块属性，便于后续动态切换）
console = console_util.console


# ------------------------------
# Computer 工厂函数类型标注
# ------------------------------
# 中文:
#  - 该类型表示一个可调用对象（函数或类构造器），用于创建 Computer 或其子类的实例。
#  - 参数签名需与 Computer.__init__ 兼容；你可以据此在你自己的工厂函数上添加类型注释。
# English:
#  - This type denotes a callable (function or class constructor) used to create a Computer or subclass instance.
#  - The parameter signature must be compatible with Computer.__init__; use it for your own factory annotations.
ComputerFactory = Callable[
    [
        set["MCPServerInputModel"] | None,
        bool,
        bool,
        Callable[[str, str, str, dict], bool] | None,
        InputResolver | None,
    ],
    Computer,
]


@app.callback(invoke_without_command=True)
def _root(
    ctx: typer.Context,
    auto_connect: bool = typer.Option(True, help="是否自动连接 / Auto connect"),
    auto_reconnect: bool = typer.Option(True, help="是否自动重连 / Auto reconnect"),
    url: str | None = typer.Option(None, help="Socket.IO 服务器URL，例如 https://host:port"),
    namespace: str = typer.Option(
        SMCP_NAMESPACE,
        "--namespace",
        help="Socket.IO 命名空间（默认: /smcp）/ Namespace to connect (default: /smcp)",
    ),
    auth: str | None = typer.Option(None, help="认证参数，形如 key:value,foo:bar"),
    headers: str | None = typer.Option(None, help="请求头参数，形如 key:value,foo:bar"),
    computer_factory: str | None = typer.Option(
        None,
        "--computer-factory",
        help=(
            "指定用于构建 Computer 的导入路径 (模块:属性 或 模块.属性)。\n"
            "例如 my_pkg.my_mod:build_computer 或 my_pkg.my_mod.MySubComputer。\n"
            "不支持以 '.' 开头的相对导入；模块解析相对于运行 a2c-computer 时的工作目录可导入包环境。"
        ),
    ),
    no_color: bool = typer.Option(
        False,
        "--no-color",
        help="关闭彩色输出（PyCharm控制台不渲染ANSI时可使用） / Disable ANSI colors",
    ),
) -> None:
    """
    根级入口：
    - 若未指定子命令，则等价于执行 `run`，保持 `a2c-computer` 和 `a2c-computer run` 两种用法都可用。
    - 若指定了子命令，则不做处理，交给子命令。
    """
    # 根据 no_color 动态调整全局 Console
    if no_color:
        global console
        console_util.set_no_color(True)
        # 重新绑定本地引用
        console = console_util.console

    if ctx.invoked_subcommand is None:
        # 注意：不要直接调用被 @app.command 装饰的 run()，否则未传入的参数会保留 OptionInfo 默认值
        # 这里改为调用纯实现函数 _run_impl，并显式传入 config=None 与 inputs=None。
        _run_impl(
            auto_connect=auto_connect,
            auto_reconnect=auto_reconnect,
            url=url,
            namespace=namespace,
            auth=auth,
            headers=headers,
            computer_factory=computer_factory,
            config=None,
            inputs=None,
        )


async def _interactive_loop(comp: Computer, init_client: SMCPComputerClient | None = None) -> None:
    """
    中文: 兼容外部引用的包装器，委托到 interactive_impl，并注入依赖。
    English: Backward-compatible wrapper that delegates to interactive_impl with dependencies injected.
    """
    await _interactive_loop_impl(
        comp,
        session_factory=PromptSession,
        patch_stdout_ctx=patch_stdout,
        smcp_client_cls=SMCPComputerClient,
        init_client=init_client,
    )


def _run_impl(
    *,
    auto_connect: bool,
    auto_reconnect: bool,
    url: str | None,
    namespace: str | None,
    auth: str | None,
    headers: str | None,
    computer_factory: str | None,
    config: str | None,
    inputs: str | None,
) -> None:
    """
    纯实现函数：不要在此处使用 Typer 的 Option 默认值，避免 OptionInfo 泄露到运行时。
    Both CLI (@app.command) 与回调 (@app.callback) 在需要时调用本函数。
    """

    async def _amain() -> None:
        # 初始化空配置，后续通过交互动态维护 / init with empty config, then manage dynamically
        # 解析工厂：默认使用 Computer 构造函数；若提供 --computer-factory，则动态导入。
        comp_factory_obj: Any = Computer
        if computer_factory:
            try:
                comp_factory_obj = resolve_import_target(computer_factory)
            except Exception as e:  # pragma: no cover
                console.print(f"[red]解析 --computer-factory 失败: {e} / Failed to resolve computer factory: {e}[/red]")
                comp_factory_obj = Computer

        # 类型提示：comp_factory_obj 应满足 ComputerFactory，可是运行时仅作 best-effort 校验
        if not callable(comp_factory_obj):  # pragma: no cover
            console.print("[red]计算机构造目标不可调用，将回退到默认 Computer[/red]")
            comp_factory_obj = Computer

        comp = comp_factory_obj(
            name="friday_hands",
            inputs=set(),
            mcp_servers=set(),
            auto_connect=auto_connect,
            auto_reconnect=auto_reconnect,
        )
        async with comp:
            init_client: SMCPComputerClient | None = None
            if url:
                try:
                    auth_dict = parse_kv_pairs(auth)
                    headers_dict = parse_kv_pairs(headers)
                except Exception as e:
                    console.print(f"[red]启动参数解析失败 / Failed to parse CLI params: {e}[/red]")
                    auth_dict = None
                    headers_dict = None
                init_client = SMCPComputerClient(computer=comp)
                # 通过 CLI 指定命名空间，确保连接时建立对应 namespace 会话
                await init_client.connect(url, auth=auth_dict, headers=headers_dict, namespaces=[namespace])
                console.print("[green]已通过启动参数连接到 Socket.IO / Connected via CLI options[/green]")

            # 启动参数加载 inputs 与 servers 配置
            # Load inputs first (so that servers config rendering can use them if needed later via interactive commands)
            if inputs:
                try:
                    ipath = inputs[1:] if inputs.startswith("@") else inputs
                    data = json.loads(Path(ipath).read_text(encoding="utf-8"))
                    # 允许单个对象或数组
                    if isinstance(data, list):
                        raw_items = TypeAdapter(list[SMCPServerInputDict]).validate_python(data)
                        models: set[MCPServerInputModel] = {TypeAdapter(MCPServerInputModel).validate_python(item) for item in raw_items}
                        comp.update_inputs(models)
                    else:
                        item: SMCPServerInputDict = TypeAdapter(SMCPServerInputDict).validate_python(data)
                        comp.add_or_update_input(TypeAdapter(MCPServerInputModel).validate_python(item))
                    console.print("[green]已加载 Inputs 配置 / Inputs loaded[/green]")
                except Exception as e:  # pragma: no cover
                    console.print(f"[red]加载 Inputs 失败 / Failed to load inputs: {e}[/red]")

            if config:
                try:
                    spath = config[1:] if config.startswith("@") else config
                    data = json.loads(Path(spath).read_text(encoding="utf-8"))
                    # 允许单个对象或数组

                    async def _add_server(cfg_obj: dict[str, Any]) -> None:
                        validated: dict[str, Any] = TypeAdapter(SMCPServerConfigDict).validate_python(cfg_obj)
                        await comp.aadd_or_aupdate_server(validated)

                    if isinstance(data, list):
                        for cfg in data:
                            await _add_server(cfg)
                    else:
                        await _add_server(data)
                    console.print("[green]已加载 Servers 配置 / Servers loaded[/green]")
                except Exception as e:  # pragma: no cover
                    console.print(f"[red]加载 Servers 失败 / Failed to load servers: {e}[/red]")

            await _interactive_loop(comp, init_client=init_client)

    asyncio.run(_amain())


@app.command()
def run(
    auto_connect: bool = typer.Option(True, help="是否自动连接 / Auto connect"),
    auto_reconnect: bool = typer.Option(True, help="是否自动重连 / Auto reconnect"),
    url: str | None = typer.Option(None, help="Socket.IO 服务器URL，例如 https://host:port"),
    namespace: str = typer.Option(
        SMCP_NAMESPACE,
        "--namespace",
        help="Socket.IO 命名空间（默认: /smcp）/ Namespace to connect (default: /smcp)",
    ),
    auth: str | None = typer.Option(None, help="认证参数，形如 key:value,foo:bar"),
    headers: str | None = typer.Option(None, help="请求头参数，形如 key:value,foo:bar"),
    computer_factory: str | None = typer.Option(
        None,
        "--computer-factory",
        help=(
            "指定用于构建 Computer 的导入路径 (模块:属性 或 模块.属性)。\n"
            "例如 my_pkg.my_mod:build_computer 或 my_pkg.my_mod.MySubComputer。\n"
            "不支持以 '.' 开头的相对导入；模块解析相对于运行 a2c-computer 时的工作目录可导入包环境。"
        ),
    ),
    config: str | None = typer.Option(
        None,
        "--config",
        "-c",
        help="在启动时从文件加载 MCP Servers 配置（支持 @file 语法或直接文件路径） / Load MCP Servers from file at startup",
    ),
    inputs: str | None = typer.Option(
        None,
        "--inputs",
        "-i",
        help="在启动时从文件加载 Inputs 定义（支持 @file 语法或直接文件路径） / Load Inputs from file at startup",
    ),
) -> None:
    """
    中文: 启动计算机并进入持续运行模式。后续将支持从配置文件加载 servers 与 inputs。
    English: Boot the computer and enter persistent loop. Config-file loading will be added later.
    """
    _run_impl(
        auto_connect=auto_connect,
        auto_reconnect=auto_reconnect,
        url=url,
        namespace=namespace,
        auth=auth,
        headers=headers,
        computer_factory=computer_factory,
        config=config,
        inputs=inputs,
    )


# 为 console_scripts 兼容提供入口
def main() -> None:  # pragma: no cover
    # 使用 Typer 应用入口，而不是直接调用命令函数
    # 直接调用被 @app.command 装饰的函数会传入 OptionInfo 默认值，导致参数类型错误
    app()



# -*- coding: utf-8 -*-
# filename: test_cli_tc_vrl.py
# @Time    : 2025/10/06 14:55
# @Author  : JQQ
# @Email   : jqq1716@gmail.com
# @Software: PyCharm
"""
中文:
  - 端到端验证交互式 CLI 的 `tc` 命令配合 VRL 转换功能。
  - 通过配置 VRL 脚本，验证工具返回值被正确转换并存储在元数据中。
  - 验证 VRL 转换结果以 JSON 格式存储在 `a2c_vrl_transformed` 字段中。

English:
  - E2E test for interactive CLI `tc` command with VRL transformation.
  - Verify tool return values are correctly transformed via VRL script.
  - Verify VRL transformation results are stored in `a2c_vrl_transformed` metadata field as JSON.

测试要点与断言:
  1) 配置 MCP Server 时带上 VRL 脚本
  2) 成功添加并启动配置了 VRL 的服务器
  3) 发送 `tc` 调用工具后，输出包含 VRL 转换结果标识
  4) 验证元数据中包含 `a2c_vrl_transformed` 字段
  5) 验证 VRL 转换后的数据结构符合预期
"""

from __future__ import annotations

import json
import sys
from pathlib import Path

import pytest

from tests.e2e.computer.utils import expect_prompt_stable, strip_ansi

pexpect = pytest.importorskip("pexpect", reason="e2e tests require pexpect; install with `pip install pexpect`.")


@pytest.mark.e2e
def test_tc_with_vrl_transformation_basic(cli_proc: pexpect.spawn, tmp_path: Path) -> None:
    """
    中文: 验证配置 VRL 脚本后，tc 调用工具返回的元数据中包含 VRL 转换结果。
    English: Verify tc tool call returns VRL transformation result in metadata when VRL is configured.
    """
    child = cli_proc

    # 1) 配置 VRL 脚本：添加一个新字段 transformed_by_vrl = true
    # Configure VRL script: add a new field transformed_by_vrl = true
    vrl_script = '.transformed_by_vrl = true\n.vrl_test_field = "e2e_test"'

    # 2) 写入 server 配置文件，包含 VRL 脚本
    # Write server config file with VRL script
    server_cfg = {
        "name": "e2e-tc-vrl-basic",
        "type": "stdio",
        "disabled": False,
        "forbidden_tools": [],
        "tool_meta": {"hello": {"auto_apply": True}},
        "vrl": vrl_script,  # 添加 VRL 配置 / Add VRL config
        "server_parameters": {
            "command": sys.executable,
            "args": [
                "tests/integration_tests/computer/mcp_servers/direct_execution.py",
            ],
            "env": None,
            "cwd": None,
            "encoding": "utf-8",
            "encoding_error_handler": "strict",
        },
    }
    cfg_path = tmp_path / "server_e2e_tc_vrl_basic.json"
    cfg_path.write_text(json.dumps(server_cfg, ensure_ascii=False), encoding="utf-8")

    # 3) 添加配置并启动
    # Add config and start server
    child.sendline(f"server add @{cfg_path}")
    expect_prompt_stable(child, quiet=0.6, max_wait=15.0)
    child.sendline("start e2e-tc-vrl-basic")
    expect_prompt_stable(child, quiet=0.6, max_wait=15.0)

    # 4) 确认工具已可见
    # Verify tool is visible
    child.sendline("tools")
    tools_out = expect_prompt_stable(child, quiet=0.6, max_wait=12.0)
    assert "hello" in strip_ansi(tools_out)

    # 5) 构造 tc 负载并调用
    # Construct tc payload and call
    tc_payload = {
        "agent": "bot-e2e-vrl",
        "req_id": "req-e2e-vrl-hello",
        "computer": "ignored",
        "tool_name": "hello",
        "params": {"name": "VRL-Test"},
        "timeout": 10,
    }

    child.sendline(f"tc {json.dumps(tc_payload, ensure_ascii=False)}")
    out = expect_prompt_stable(child, quiet=0.8, max_wait=20.0)
    out = strip_ansi(out)

    # 6) 断言包含工具调用结果
    # Assert tool call result is present
    assert "Hello, VRL-Test!" in out, f"unexpected tc output:\n{out}"

    # 7) 断言包含 VRL 转换标识
    # Assert VRL transformation marker is present
    assert "a2c_vrl_transformed" in out, f"VRL transformation marker not found in output:\n{out}"

    # 8) 验证 VRL 转换后的字段存在
    # Verify VRL transformed fields exist
    assert "transformed_by_vrl" in out or "vrl_test_field" in out, f"VRL transformed fields not found in output:\n{out}"


@pytest.mark.e2e
def test_tc_with_vrl_field_mapping(cli_proc: pexpect.spawn, tmp_path: Path) -> None:
    """
    中文: 验证 VRL 可以对工具返回值进行字段映射和数据转换。
    English: Verify VRL can perform field mapping and data transformation on tool return values.
    """
    child = cli_proc

    # 1) 配置 VRL 脚本：提取工具返回的文本内容，并重新组织结构
    # Configure VRL script: extract text content and reorganize structure
    vrl_script = """
.status = "success"
.original_content = .content[0].text
.metadata = {
    "processed": true,
    "processor": "vrl-e2e-test"
}
"""

    # 2) 写入 server 配置文件
    # Write server config file
    server_cfg = {
        "name": "e2e-tc-vrl-mapping",
        "type": "stdio",
        "disabled": False,
        "forbidden_tools": [],
        "tool_meta": {"mark_a": {"auto_apply": True}},
        "vrl": vrl_script,
        "server_parameters": {
            "command": sys.executable,
            "args": [
                "tests/integration_tests/computer/mcp_servers/resources_subscribe_stdio_server.py",
            ],
            "env": None,
            "cwd": None,
            "encoding": "utf-8",
            "encoding_error_handler": "strict",
        },
    }
    cfg_path = tmp_path / "server_e2e_tc_vrl_mapping.json"
    cfg_path.write_text(json.dumps(server_cfg, ensure_ascii=False), encoding="utf-8")

    # 3) 添加配置并启动
    # Add config and start server
    child.sendline(f"server add @{cfg_path}")
    expect_prompt_stable(child, quiet=0.6, max_wait=15.0)
    child.sendline("start e2e-tc-vrl-mapping")
    expect_prompt_stable(child, quiet=0.6, max_wait=15.0)

    # 4) 确认工具已可见
    # Verify tool is visible
    child.sendline("tools")
    tools_out = expect_prompt_stable(child, quiet=0.6, max_wait=12.0)
    assert "mark_a" in strip_ansi(tools_out)

    # 5) 构造 tc 负载并调用
    # Construct tc payload and call
    tc_payload = {
        "agent": "bot-e2e-vrl-map",
        "req_id": "req-e2e-vrl-map",
        "computer": "ignored",
        "tool_name": "mark_a",
        "params": {},
        "timeout": 10,
    }

    child.sendline(f"tc {json.dumps(tc_payload, ensure_ascii=False)}")
    out = expect_prompt_stable(child, quiet=0.8, max_wait=20.0)
    out = strip_ansi(out)

    # 6) 断言包含工具调用结果
    # Assert tool call result is present
    assert "ok:mark_a" in out, f"unexpected tc output:\n{out}"

    # 7) 断言包含 VRL 转换标识和转换后的字段
    # Assert VRL transformation marker and transformed fields are present
    assert "a2c_vrl_transformed" in out, f"VRL transformation marker not found in output:\n{out}"
    # VRL转换结果在meta中是JSON字符串，可能被转义，所以检查转义后的形式
    # VRL transformation result in meta is JSON string, may be escaped, so check escaped form
    assert '"status"' in out or "'status'" in out or '\\"status\\"' in out, f"VRL transformed 'status' field not found in output:\n{out}"
    assert '"success"' in out or "'success'" in out or '\\"success\\"' in out, (
        f"VRL transformed 'success' value not found in output:\n{out}"
    )


@pytest.mark.e2e
def test_tc_with_invalid_vrl_syntax_rejected(cli_proc: pexpect.spawn, tmp_path: Path) -> None:
    """
    中文: 验证配置了无效 VRL 语法的服务器会在添加时被拒绝。
    English: Verify server with invalid VRL syntax is rejected during configuration.
    """
    child = cli_proc

    # 1) 配置无效的 VRL 脚本（语法错误）
    # Configure invalid VRL script (syntax error)
    invalid_vrl_script = ".invalid syntax here @@@ this will fail"

    # 2) 写入 server 配置文件
    # Write server config file
    server_cfg = {
        "name": "e2e-tc-vrl-invalid",
        "type": "stdio",
        "disabled": False,
        "forbidden_tools": [],
        "tool_meta": {"hello": {"auto_apply": True}},
        "vrl": invalid_vrl_script,  # 无效的 VRL 脚本 / Invalid VRL script
        "server_parameters": {
            "command": sys.executable,
            "args": [
                "tests/integration_tests/computer/mcp_servers/direct_execution.py",
            ],
            "env": None,
            "cwd": None,
            "encoding": "utf-8",
            "encoding_error_handler": "strict",
        },
    }
    cfg_path = tmp_path / "server_e2e_tc_vrl_invalid.json"
    cfg_path.write_text(json.dumps(server_cfg, ensure_ascii=False), encoding="utf-8")

    # 3) 尝试添加配置，应该失败
    # Try to add config, should fail
    child.sendline(f"server add @{cfg_path}")
    out = expect_prompt_stable(child, quiet=0.6, max_wait=15.0)
    out = strip_ansi(out)

    # 4) 断言包含错误信息
    # Assert error message is present
    assert "VRL" in out or "语法错误" in out or "syntax error" in out or "error" in out.lower(), (
        f"Expected VRL syntax error message in output:\n{out}"
    )


@pytest.mark.e2e
def test_tc_vrl_transformation_preserves_original_content(cli_proc: pexpect.spawn, tmp_path: Path) -> None:
    """
    中文: 验证 VRL 转换不影响原始工具返回内容，转换结果仅存储在元数据中。
    English: Verify VRL transformation doesn't affect original tool return content,
              transformation result is only stored in metadata.
    """
    child = cli_proc

    # 1) 配置 VRL 脚本
    # Configure VRL script
    vrl_script = '.vrl_marker = "transformed"\n.extra_field = 42'

    # 2) 写入 server 配置文件
    # Write server config file
    server_cfg = {
        "name": "e2e-tc-vrl-preserve",
        "type": "stdio",
        "disabled": False,
        "forbidden_tools": [],
        "tool_meta": {"hello": {"auto_apply": True}},
        "vrl": vrl_script,
        "server_parameters": {
            "command": sys.executable,
            "args": [
                "tests/integration_tests/computer/mcp_servers/direct_execution.py",
            ],
            "env": None,
            "cwd": None,
            "encoding": "utf-8",
            "encoding_error_handler": "strict",
        },
    }
    cfg_path = tmp_path / "server_e2e_tc_vrl_preserve.json"
    cfg_path.write_text(json.dumps(server_cfg, ensure_ascii=False), encoding="utf-8")

    # 3) 添加配置并启动
    # Add config and start server
    child.sendline(f"server add @{cfg_path}")
    expect_prompt_stable(child, quiet=0.6, max_wait=15.0)
    child.sendline("start e2e-tc-vrl-preserve")
    expect_prompt_stable(child, quiet=0.6, max_wait=15.0)

    # 4) 构造 tc 负载并调用
    # Construct tc payload and call
    tc_payload = {
        "agent": "bot-e2e-vrl-preserve",
        "req_id": "req-e2e-vrl-preserve",
        "computer": "ignored",
        "tool_name": "hello",
        "params": {"name": "Preserve-Test"},
        "timeout": 10,
    }

    child.sendline(f"tc {json.dumps(tc_payload, ensure_ascii=False)}")
    out = expect_prompt_stable(child, quiet=0.8, max_wait=20.0)
    out = strip_ansi(out)

    # 5) 断言原始内容仍然存在
    # Assert original content is still present
    assert "Hello, Preserve-Test!" in out, f"Original content missing in output:\n{out}"

    # 6) 断言 VRL 转换结果也存在于元数据中
    # Assert VRL transformation result exists in metadata
    assert "a2c_vrl_transformed" in out, f"VRL transformation marker not found in output:\n{out}"
    assert "vrl_marker" in out or "extra_field" in out, f"VRL transformed fields not found in output:\n{out}"



"""
文件名: resolver.py
作者: JQQ
创建日期: 2025/9/18
最后修改日期: 2025/9/18
版权: 2023 JQQ. All rights reserved.
依赖: prompt_toolkit, rich
描述:
  中文: inputs 解析器定义与实现。按需根据 id 解析三类输入：promptString、pickString、command。
  English: Input resolvers. Lazily resolve three kinds of inputs by id: promptString, pickString, command.
"""

from __future__ import annotations

from collections.abc import Iterable
from typing import Any

from prompt_toolkit import PromptSession

from a2c_smcp.computer.inputs.base import BaseInputResolver
from a2c_smcp.computer.inputs.cli_io import ainput_pick, ainput_prompt, arun_command
from a2c_smcp.computer.mcp_clients.model import (
    MCPServerCommandInput,
    MCPServerInput,
    MCPServerPickStringInput,
    MCPServerPromptStringInput,
)
from a2c_smcp.utils.logger import logger


class InputNotFoundError(KeyError):
    pass


class InputResolver(BaseInputResolver[PromptSession]):
    """
    中文: 输入解析器，支持基于 id 的惰性解析与结果缓存。
    English: Input resolver with lazy per-id resolution and result cache.
    """

    def __init__(self, inputs: Iterable[MCPServerInput], session: PromptSession | None = None) -> None:
        """
        中文: CLI 特化的输入解析器，支持可选的 PromptSession 注入。
        English: CLI-specialized input resolver with optional PromptSession injection.
        """
        super().__init__(inputs, session=session)

    async def aresolve_by_id(self, input_id: str, *, session: PromptSession | None = None) -> Any:
        if input_id in self._cache:
            return self._cache[input_id]
        cfg = self._inputs.get(input_id)
        if not cfg:
            raise InputNotFoundError(input_id)

        # 优先使用调用时传入的 session，其次使用初始化指定的 self.session
        # Prefer provided session; fallback to self.session
        sess = session or self.session

        if isinstance(cfg, MCPServerPromptStringInput):
            value = await self._aresolve_prompt(cfg, session=sess)
        elif isinstance(cfg, MCPServerPickStringInput):
            value = await self._aresolve_pick(cfg, session=sess)
        elif isinstance(cfg, MCPServerCommandInput):
            value = await self._aresolve_command(cfg)
        else:  # pragma: no cover
            logger.warning(f"未知输入类型: {type(cfg)} / Unknown input type")
            value = None

        self._cache[input_id] = value
        return value

    async def _aresolve_prompt(self, cfg: MCPServerPromptStringInput, *, session: PromptSession | None = None) -> str:
        msg = cfg.description or f"请输入 {cfg.id} / Please input {cfg.id}"
        pwd = bool(cfg.password)
        return await ainput_prompt(msg, password=pwd, default=cfg.default, session=session)

    async def _aresolve_pick(self, cfg: MCPServerPickStringInput, *, session: PromptSession | None = None) -> str:
        msg = cfg.description or f"请选择 {cfg.id} / Please pick {cfg.id}"
        options = cfg.options or []
        default_index = None
        if cfg.default is not None and cfg.default in options:
            default_index = options.index(cfg.default)
        picked = await ainput_pick(msg, options, default_index=default_index, multi=False, session=session)
        return str(picked) or (cfg.default or "")

    async def _aresolve_command(self, cfg: MCPServerCommandInput) -> Any:
        # 约定: command 为完整可执行字符串，由 shell 执行。args 如存在，暂不拼接，后续可扩展。
        return await arun_command(cfg.command, shell=True, parse="raw")



# -*- coding: utf-8 -*-
# filename: conftest.py
# @Time    : 2025/8/20 14:03
# @Author  : JQQ
# @Email   : jqq1716@gmail.com
# @Software: PyCharm
import multiprocessing
import socket
import sys
import time
from collections.abc import Callable, Generator
from datetime import timedelta
from pathlib import Path

import anyio
import pytest
import uvicorn
from mcp import ErrorData, McpError, StdioServerParameters, Tool
from mcp import types as types
from mcp.client.session_group import SseServerParameters, StreamableHttpParameters
from mcp.server import Server
from mcp.server.sse import SseServerTransport
from mcp.server.streamable_http import EventCallback, EventId, EventMessage, EventStore, StreamId
from mcp.server.streamable_http_manager import StreamableHTTPSessionManager
from mcp.server.transport_security import TransportSecuritySettings
from mcp.types import TextContent
from pydantic import AnyUrl
from starlette.applications import Starlette
from starlette.requests import Request
from starlette.responses import Response
from starlette.routing import Mount, Route

from a2c_smcp.computer.mcp_clients.http_client import HttpMCPClient

STREAMABLE_HTTP_SERVER_NAME = "test_streamable_http_server"
TEST_SESSION_ID = "test-session-id-12345"
INIT_REQUEST = {
    "jsonrpc": "2.0",
    "method": "initialize",
    "params": {
        "clientInfo": {"name": "test-client", "version": "1.0"},
        "protocolVersion": "2025-03-26",
        "capabilities": {},
    },
    "id": "init-1",
}


class SimpleEventStore(EventStore):
    """Simple in-memory event store for testing."""

    def __init__(self):
        self._events: list[tuple[StreamId, EventId, types.JSONRPCMessage]] = []
        self._event_id_counter = 0

    async def store_event(self, stream_id: StreamId, message: types.JSONRPCMessage) -> EventId:
        """Store an event and return its ID."""
        self._event_id_counter += 1
        event_id = str(self._event_id_counter)
        self._events.append((stream_id, event_id, message))
        return event_id

    async def replay_events_after(
        self,
        last_event_id: EventId,
        send_callback: EventCallback,
    ) -> StreamId | None:
        """Replay events after the specified ID."""
        # Find the stream ID of the last event
        target_stream_id = None
        for stream_id, event_id, _ in self._events:
            if event_id == last_event_id:
                target_stream_id = stream_id
                break

        if target_stream_id is None:
            # If event ID not found, return None
            return None

        # Convert last_event_id to int for comparison
        last_event_id_int = int(last_event_id)

        # Replay only events from the same stream with ID > last_event_id
        for stream_id, event_id, message in self._events:
            if stream_id == target_stream_id and int(event_id) > last_event_id_int:
                await send_callback(EventMessage(message, event_id))

        return target_stream_id


class StreamableHttpServerTest(Server):
    def __init__(self):
        super().__init__(STREAMABLE_HTTP_SERVER_NAME)
        self._lock = None  # Will be initialized in async context

        @self.read_resource()
        async def handle_read_resource(uri: AnyUrl) -> str | bytes:
            if uri.scheme == "foobar":
                return f"Read {uri.host}"
            elif uri.scheme == "slow":
                # Simulate a slow resource
                await anyio.sleep(2.0)
                return f"Slow response from {uri.host}"

            raise ValueError(f"Unknown resource: {uri}")

        @self.list_tools()
        async def handle_list_tools() -> list[Tool]:
            return [
                Tool(
                    name="test_tool",
                    description="A test tool",
                    inputSchema={"type": "object", "properties": {}},
                ),
                Tool(
                    name="test_tool_with_standalone_notification",
                    description="A test tool that sends a notification",
                    inputSchema={"type": "object", "properties": {}},
                ),
                Tool(
                    name="long_running_with_checkpoints",
                    description="A long-running tool that sends periodic notifications",
                    inputSchema={"type": "object", "properties": {}},
                ),
                Tool(
                    name="test_sampling_tool",
                    description="A tool that triggers server-side sampling",
                    inputSchema={"type": "object", "properties": {}},
                ),
                Tool(
                    name="wait_for_lock_with_notification",
                    description="A tool that sends a notification and waits for lock",
                    inputSchema={"type": "object", "properties": {}},
                ),
                Tool(
                    name="release_lock",
                    description="A tool that releases the lock",
                    inputSchema={"type": "object", "properties": {}},
                ),
                Tool(
                    name="trigger_list_changed",
                    description="Trigger tools/resources/prompts listChanged notifications",
                    inputSchema={"type": "object", "properties": {}},
                ),
            ]

        @self.call_tool()
        async def handle_call_tool(name: str, args: dict) -> list[TextContent]:
            ctx = self.request_context

            # When the tool is called, send a notification to test GET stream
            if name == "test_tool_with_standalone_notification":
                await ctx.session.send_resource_updated(uri=AnyUrl("http://test_resource"))
                return [TextContent(type="text", text=f"Called {name}")]

            elif name == "long_running_with_checkpoints":
                # Send notifications that are part of the response stream
                # This simulates a long-running tool that sends logs

                await ctx.session.send_log_message(
                    level="info",
                    data="Tool started",
                    logger="tool",
                    related_request_id=ctx.request_id,  # need for stream association
                )

                await anyio.sleep(0.1)

                await ctx.session.send_log_message(
                    level="info",
                    data="Tool is almost done",
                    logger="tool",
                    related_request_id=ctx.request_id,
                )

                return [TextContent(type="text", text="Completed!")]

            elif name == "test_sampling_tool":
                # Test sampling by requesting the client to sample a message
                sampling_result = await ctx.session.create_message(
                    messages=[
                        types.SamplingMessage(
                            role="user",
                            content=types.TextContent(type="text", text="Server needs client sampling"),
                        ),
                    ],
                    max_tokens=100,
                    related_request_id=ctx.request_id,
                )

                # Return the sampling result in the tool response
                response = sampling_result.content.text if sampling_result.content.type == "text" else None
                return [
                    TextContent(
                        type="text",
                        text=f"Response from sampling: {response}",
                    ),
                ]

            elif name == "wait_for_lock_with_notification":
                # Initialize lock if not already done
                if self._lock is None:
                    self._lock = anyio.Event()

                # First send a notification
                await ctx.session.send_log_message(
                    level="info",
                    data="First notification before lock",
                    logger="lock_tool",
                    related_request_id=ctx.request_id,
                )

                # Now wait for the lock to be released
                await self._lock.wait()

                # Send second notification after lock is released
                await ctx.session.send_log_message(
                    level="info",
                    data="Second notification after lock",
                    logger="lock_tool",
                    related_request_id=ctx.request_id,
                )

                return [TextContent(type="text", text="Completed")]

            elif name == "release_lock":
                assert self._lock is not None, "Lock must be initialized before releasing"

                # Release the lock
                self._lock.set()
                return [TextContent(type="text", text="Lock released")]
            elif name == "trigger_list_changed":
                # 发送列表变更通知 / send list-changed notifications
                await ctx.session.send_tool_list_changed()
                await ctx.session.send_resource_list_changed()
                await ctx.session.send_prompt_list_changed()
                return [TextContent(type="text", text="changes triggered")]

            elif name == "nonexistent_tool":
                raise McpError(
                    error=types.ErrorData(code=404, message="OOPS! no tool with that name was found"),
                )

            return [TextContent(type="text", text=f"Called {name}")]


def create_app(is_json_response_enabled=False, event_store: EventStore | None = None) -> Starlette:
    """Create a Starlette application for testing using the session manager.

    Args:
        is_json_response_enabled: If True, use JSON responses instead of SSE streams.
        event_store: Optional event store for testing resumability.
    """
    # Create server instance
    server = StreamableHttpServerTest()

    # Create the session manager
    security_settings = TransportSecuritySettings(
        allowed_hosts=["127.0.0.1:*", "localhost:*"],
        allowed_origins=["http://127.0.0.1:*", "http://localhost:*"],
    )
    session_manager = StreamableHTTPSessionManager(
        app=server,
        event_store=event_store,
        json_response=is_json_response_enabled,
        security_settings=security_settings,
    )

    # Create an ASGI application that uses the session manager
    app = Starlette(
        debug=True,
        routes=[
            Mount("/mcp", app=session_manager.handle_request),
        ],
        lifespan=lambda app: session_manager.run(),  # type: ignore
    )

    return app


def run_streamable_http_server(port: int, is_json_response_enabled=False, event_store: EventStore | None = None) -> None:
    """Run the test streamable HTTP server.

    Args:
        port: Port to listen on.
        is_json_response_enabled: If True, use JSON responses instead of SSE streams.
        event_store: Optional event store for testing resumability.
    """

    app = create_app(is_json_response_enabled, event_store)
    # Configure server
    config = uvicorn.Config(
        app=app,
        host="127.0.0.1",
        port=port,
        log_level="info",
        limit_concurrency=10,
        timeout_keep_alive=5,
        access_log=False,
    )

    # Start the server
    server = uvicorn.Server(config=config)

    # This is important to catch exceptions and prevent test hangs
    try:
        server.run()
    except Exception:
        import traceback

        traceback.print_exc()


@pytest.fixture
def basic_server_port() -> int:
    """Find an available port for the basic server."""
    with socket.socket() as s:
        s.bind(("127.0.0.1", 0))
        return s.getsockname()[1]


@pytest.fixture
def json_server_port() -> int:
    """Find an available port for the JSON response server."""
    with socket.socket() as s:
        s.bind(("127.0.0.1", 0))
        return s.getsockname()[1]


@pytest.fixture
def basic_server(basic_server_port: int) -> Generator[None, None, None]:
    """Start a basic server."""
    proc = multiprocessing.Process(target=run_streamable_http_server, kwargs={"port": basic_server_port}, daemon=True)
    proc.start()

    # Wait for server to be running
    max_attempts = 20
    attempt = 0
    while attempt < max_attempts:
        try:
            with socket.socket(socket.AF_INET, socket.SOCK_STREAM) as s:
                s.connect(("127.0.0.1", basic_server_port))
                break
        except ConnectionRefusedError:
            time.sleep(0.1)
            attempt += 1
    else:
        raise RuntimeError(f"Server failed to start after {max_attempts} attempts")

    yield

    # Clean up
    proc.kill()
    proc.join(timeout=2)


@pytest.fixture
def event_store() -> SimpleEventStore:
    """Create a test event store."""
    return SimpleEventStore()


@pytest.fixture
def event_server_port() -> int:
    """Find an available port for the event store server."""
    with socket.socket() as s:
        s.bind(("127.0.0.1", 0))
        return s.getsockname()[1]


@pytest.fixture
def event_server(event_server_port: int, event_store: SimpleEventStore) -> Generator[tuple[SimpleEventStore, str], None, None]:
    """Start a server with event store enabled."""
    proc = multiprocessing.Process(
        target=run_streamable_http_server,
        kwargs={"port": event_server_port, "event_store": event_store},
        daemon=True,
    )
    proc.start()

    # Wait for server to be running
    max_attempts = 20
    attempt = 0
    while attempt < max_attempts:
        try:
            with socket.socket(socket.AF_INET, socket.SOCK_STREAM) as s:
                s.connect(("127.0.0.1", event_server_port))
                break
        except ConnectionRefusedError:
            time.sleep(0.1)
            attempt += 1
    else:
        raise RuntimeError(f"Server failed to start after {max_attempts} attempts")

    yield event_store, f"http://127.0.0.1:{event_server_port}"

    # Clean up
    proc.kill()
    proc.join(timeout=2)


@pytest.fixture
def json_response_server(json_server_port: int) -> Generator[None, None, None]:
    """Start a server with JSON response enabled."""
    proc = multiprocessing.Process(
        target=run_streamable_http_server,
        kwargs={"port": json_server_port, "is_json_response_enabled": True},
        daemon=True,
    )
    proc.start()

    # Wait for server to be running
    max_attempts = 20
    attempt = 0
    while attempt < max_attempts:
        try:
            with socket.socket(socket.AF_INET, socket.SOCK_STREAM) as s:
                s.connect(("127.0.0.1", json_server_port))
                break
        except ConnectionRefusedError:
            time.sleep(0.1)
            attempt += 1
    else:
        raise RuntimeError(f"Server failed to start after {max_attempts} attempts")

    yield

    # Clean up
    proc.kill()
    proc.join(timeout=2)


@pytest.fixture
def basic_server_url(basic_server_port: int) -> str:
    """Get the URL for the basic test server."""
    return f"http://127.0.0.1:{basic_server_port}"


@pytest.fixture
def json_server_url(json_server_port: int) -> str:
    """Get the URL for the JSON response test server."""
    return f"http://127.0.0.1:{json_server_port}"


@pytest.fixture
def http_params(basic_server_url: str) -> StreamableHttpParameters:
    """
    # 根据fixture动态生成StreamableHttpParameters，指向运行时server
    # Dynamically generate StreamableHttpParameters for runtime server
    """
    return StreamableHttpParameters(
        url=f"{basic_server_url}/mcp",
        timeout=timedelta(seconds=20),
    )


@pytest.fixture
def http_client(http_params: StreamableHttpParameters) -> HttpMCPClient:
    """
    # 创建HttpMCPClient实例
    # Create HttpMCPClient instance
    """
    return HttpMCPClient(http_params)


SSE_SERVER_NAME = "test_server_for_SSE"


@pytest.fixture(scope="session")
def server_port() -> int:
    with socket.socket() as s:
        s.bind(("127.0.0.1", 0))
        return s.getsockname()[1]


@pytest.fixture(scope="session")
def server_url(server_port: int) -> str:
    return f"http://127.0.0.1:{server_port}"


class SseServerTest(Server):
    def __init__(self) -> None:
        super().__init__(SSE_SERVER_NAME)

        @self.read_resource()
        async def handle_read_resource(uri: AnyUrl) -> str | bytes:
            if uri.scheme == "foobar":
                return f"Read {uri.host}"
            elif uri.scheme == "slow":
                # Simulate a slow resource
                await anyio.sleep(2.0)
                return f"Slow response from {uri.host}"

            raise McpError(error=ErrorData(code=404, message="OOPS! no resource with that URI was found"))

        @self.list_tools()
        async def handle_list_tools() -> list[Tool]:
            return [
                Tool(
                    name="test_tool",
                    description="A test tool",
                    inputSchema={"type": "object", "properties": {}},
                ),
                Tool(
                    name="trigger_list_changed",
                    description="Trigger tools/resources/prompts listChanged notifications",
                    inputSchema={"type": "object", "properties": {}},
                ),
            ]

        @self.call_tool()
        async def handle_call_tool(name: str, args: dict) -> list[TextContent]:
            if name == "test_tool":
                return [TextContent(type="text", text=f"Called {name}")]
            elif name == "trigger_list_changed":
                ctx = self.request_context
                await ctx.session.send_tool_list_changed()
                await ctx.session.send_resource_list_changed()
                await ctx.session.send_prompt_list_changed()
                return [TextContent(type="text", text="changes triggered")]
            else:
                raise McpError(error=ErrorData(code=404, message="OOPS! no tool with that name was found"))


def make_server_app() -> Starlette:
    """创建测试 Starlette app，带有 SSE 传输\nCreate test Starlette app with SSE transport"""
    # 配置测试安全设置 Configure security for testing
    security_settings: TransportSecuritySettings = TransportSecuritySettings(
        allowed_hosts=["127.0.0.1:*", "localhost:*"],
        allowed_origins=["http://127.0.0.1:*", "http://localhost:*"],
    )
    sse: SseServerTransport = SseServerTransport("/messages/", security_settings=security_settings)
    server: SseServerTest = SseServerTest()

    async def handle_sse(request: Request) -> Response:
        async with sse.connect_sse(request.scope, request.receive, request._send) as streams:
            await server.run(streams[0], streams[1], server.create_initialization_options())
        return Response()

    app: Starlette = Starlette(
        routes=[
            Route("/sse", endpoint=handle_sse),
            Mount("/messages/", app=sse.handle_post_message),
        ],
    )

    return app


def run_sse_server(server_port: int) -> None:
    app: Starlette = make_server_app()
    server = uvicorn.Server(config=uvicorn.Config(app=app, host="127.0.0.1", port=server_port, log_level="error"))
    print(f"starting server on {server_port}")
    server.run()

    # Give server time to start
    while not server.started:
        print("waiting for server to start")
        time.sleep(0.5)


@pytest.fixture(scope="session")
def sse_server(server_port: int) -> Generator[None, None, None]:
    proc: multiprocessing.Process = multiprocessing.Process(target=run_sse_server, kwargs={"server_port": server_port}, daemon=True)
    print("starting process")
    proc.start()

    # Wait for server to be running
    max_attempts: int = 20
    attempt: int = 0
    print("waiting for server to start")
    while attempt < max_attempts:
        try:
            with socket.socket(socket.AF_INET, socket.SOCK_STREAM) as s:
                s.connect(("127.0.0.1", server_port))
                break
        except ConnectionRefusedError:
            time.sleep(0.1)
            attempt += 1
    else:
        raise RuntimeError(f"Server failed to start after {max_attempts} attempts")

    yield

    print("killing server")
    # Signal the server to stop
    proc.kill()
    proc.join(timeout=2)
    if proc.is_alive():
        print("server process failed to terminate")


@pytest.fixture
def sse_params(server_url: str) -> SseServerParameters:
    """
    根据fixture动态生成SseServerParameters，指向运行时服务
    Dynamically generate SseServerParameters based on fixture, pointing to runtime server
    """
    return SseServerParameters(url=f"{server_url}/sse")


@pytest.fixture
def track_state() -> tuple[Callable[[str, str], None], list[tuple[str, str]]]:
    """
    跟踪状态变化的辅助函数
    Helper for tracking state changes
    """
    state_history: list[tuple[str, str]] = []

    def callback(from_state: str, to_state: str) -> None:
        state_history.append((from_state, to_state))

    return callback, state_history


TEST_DIR: Path = Path(__file__).parent.parent.parent
MCP_SERVER_SCRIPT: Path = TEST_DIR / "integration_tests" / "computer" / "mcp_servers" / "direct_execution.py"


@pytest.fixture
def stdio_params() -> StdioServerParameters:
    """提供 StdioServerParameters 配置 Provide StdioServerParameters config"""
    return StdioServerParameters(command=sys.executable, args=[str(MCP_SERVER_SCRIPT)])



# -*- coding: utf-8 -*-
# filename: resources_stdio_server.py
# @Time    : 2025/10/02 17:01
# @Author  : JQQ
# @Email   : jqq1716@gmail.com
# @Software: PyCharm
"""
中文: 提供仅 Resources 能力（不支持订阅）的最小 MCP Stdio 服务器，用于 Desktop 集成测试。
英文: Minimal MCP Stdio server exposing Resources only (without subscribe) for Desktop integration tests.

文件名: resources_stdio_server.py / filename: resources_stdio_server.py
作者: JQQ / author: JQQ
创建日期: 2025/10/02 / created at: 2025/10/02
最后修改日期: 2025/10/02 / last modified at: 2025/10/02
版权: 2023 JQQ. All rights reserved. / copyright: 2023 JQQ. All rights reserved.
依赖: anyio, mcp / dependencies: anyio, mcp
描述: 暴露 window:// 资源，符合 Desktop 资源协议要求；用于测试不支持订阅的情况。
      Expose window:// resources compatible with Desktop requirements; no subscribe support.
"""

from __future__ import annotations

import anyio
import mcp.types as types
from mcp.server.lowlevel.server import Server
from mcp.server.stdio import stdio_server

# 预置若干窗口资源 / preset window resources
WINDOW_HOST = "example.desktop.itest"
WINDOW_RESOURCES: list[types.Resource] = [
    types.Resource(
        uri=f"window://{WINDOW_HOST}/main?priority=80",
        name="Main Window",
        description="中文: 主窗口; 英文: Main window",
        mimeType="text/markdown",
    ),
    types.Resource(
        uri=f"window://{WINDOW_HOST}/secondary?priority=20",
        name="Secondary Window",
        description="中文: 次级窗口; 英文: Secondary window",
        mimeType="text/markdown",
    ),
    types.Resource(
        uri=f"window://{WINDOW_HOST}/full?priority=100&fullscreen=true",
        name="Fullscreen Window",
        description="中文: 全屏窗口; 英文: Fullscreen window",
        mimeType="text/markdown",
    ),
]


async def run() -> None:
    """
    中文: 启动仅支持 Resources 的服务器，提供 window:// 资源枚举与读取能力。
    英文: Start a server supporting only Resources, providing window:// listing and reading.
    """
    server = Server(name="itest-resources-only", version="0.0.1", instructions="itest-desktop")

    # 列举资源（分页模拟）/ list resources with pagination simulation
    @server.list_resources()
    async def list_resources(req: types.ListResourcesRequest | None = None) -> list[types.Resource]:
        return WINDOW_RESOURCES

    # 读取资源内容 / read resource contents
    @server.read_resource()
    async def read_resource(uri: types.AnyUrl):
        text = f"# Window Resource\n\nURI: {uri}\n\n中文: 这是测试用窗口内容。\n英文: This is test window content.\n"
        # 直接返回文本，由底层封装为 TextResourceContents / return str, lowlevel will wrap
        return text

    async with stdio_server() as (read_stream, write_stream):
        init_opts = server.create_initialization_options()
        await server.run(read_stream, write_stream, init_opts)


if __name__ == "__main__":
    anyio.run(run)



"""
文件名: test_cli_interactive.py
作者: JQQ
创建日期: 2025/9/22
最后修改日期: 2025/9/22
版权: 2023 JQQ. All rights reserved.
依赖: pytest, pexpect
描述:
  中文: 通过 pexpect 驱动真实进程的交互式 CLI e2e 测试用例。
  English: End-to-end tests for interactive CLI driven by pexpect against a real process.
"""

from __future__ import annotations

import re

import pytest

from tests.e2e.computer.utils import ANSI, PROMPT_RE, expect_prompt_stable, strip_ansi

pexpect = pytest.importorskip("pexpect", reason="e2e tests require pexpect; install with `pip install pexpect`.")

# 中文: 帮助标题的匹配，沿用 ANSI 感知的模式以避免误匹配 re.S(或re.DOTALL) 模式下 . 将匹配包括换行模式符在内的任意字符。因此必须强制关闭
# re.S 否则会拦截正常输出
# English: Help title regex using ANSI-aware pattern to avoid mismatches
HELP_TITLE_RE = re.compile(ANSI + r".*可用命令 / Commands.*")


@pytest.mark.e2e
def test_enter_does_nothing(cli_proc: pexpect.spawn) -> None:
    """
    中文: 在 a2c> 下按回车，应该不打印帮助，直接返回到下一次提示符。
    English: Pressing Enter at a2c> should do nothing and return to the next prompt without printing help.
    """
    child = cli_proc

    # 发送空回车并等待稳定提示符 / send empty Enter then wait for stable prompt
    child.sendline("")
    output = expect_prompt_stable(child, quiet=0.3, max_wait=10.0)

    # 不应出现帮助标题 / should not contain help title
    assert "可用命令 / Commands" not in output
    assert re.search(r"\bCommands\b", output) is None


@pytest.mark.e2e
def test_help_shows_table(cli_proc: pexpect.spawn) -> None:
    """
    中文: 输入 help 或 ? 能展示帮助表格。
    English: Typing help or ? should display the help table.
    """
    child = cli_proc

    # 请求帮助 / ask for help
    child.sendline("help")
    # 先等待帮助标题出现，再等待稳定提示符，避免抓空 / wait help title then stable prompt
    child.expect(HELP_TITLE_RE, timeout=5)
    child.expect(PROMPT_RE, timeout=5)
    output = strip_ansi((child.before or "").strip())

    assert "server add <json|@file>" in output
    assert "socket connect" in output

    # 再用 ? 验证一次 / verify with ? again
    child.sendline("?")
    child.expect(HELP_TITLE_RE)
    child.expect(PROMPT_RE, timeout=5)
    output2 = strip_ansi((child.before or "").strip())
    assert "server add <json|@file>" in output2



# -*- coding: utf-8 -*-
# filename: resources_read_error_stdio_server.py
# @Time    : 2025/10/02 22:45
# @Author  : JQQ
# @Email   : jqq1716@gmail.com
# @Software: PyCharm
"""
中文: 列举一个 window:// 资源，但在 read_resource 时抛出异常，用于覆盖 BaseMCPClient.get_window_detail 的异常路径。
英文: List a window:// resource but raise in read_resource to cover BaseMCPClient.get_window_detail exception path.
"""

from __future__ import annotations

import anyio
from mcp import types
from mcp.server.lowlevel.server import Server
from mcp.server.stdio import stdio_server

HOST = "example.desktop.readerr"
RES = types.Resource(
    uri=f"window://{HOST}/err?priority=1",
    name="ErrWin",
    description="中文: 出错窗口; 英文: error window",
    mimeType="text/markdown",
)


async def run() -> None:
    server = Server(name="itest-resources-readerr", version="0.0.1", instructions="itest-desktop")

    @server.list_resources()
    async def list_resources() -> list[types.Resource]:
        return [RES]

    @server.read_resource()
    async def read_resource(uri: types.AnyUrl):  # noqa: ARG001
        raise RuntimeError("boom")

    async with stdio_server() as (read_stream, write_stream):
        init_opts = server.create_initialization_options()
        await server.run(read_stream, write_stream, init_opts)


if __name__ == "__main__":
    anyio.run(run)



# -*- coding: utf-8 -*-
# filename: test_base_client_edge_cases.py
# @Time    : 2025/10/02 22:55
# @Author  : JQQ
# @Email   : jqq1716@gmail.com
# @Software: PyCharm
"""
中文: 覆盖 BaseMCPClient 中边界/异常分支的集成测试。
英文: Integration tests to cover edge/error branches in BaseMCPClient.

目标覆盖行 (base_client.py):
- 213-216: _keep_alive_task 异常路径 / exception path
- 351-352: list_tools 分页 / list_tools pagination
- 379-380: list_resources 分页 / list_resources pagination
- 387: 过滤非 window 资源 / filter non-window resources
- 399-401: list_windows 异常返回 [] / error path returns []
- 423: get_window_detail 接受 str / accepts str
- 426-428: get_window_detail 异常返回固定内容 / error return with fixed content
- 458-461: _close_task 等待子任务报错分支 / awaiting subtask raises non-cancel error
"""

from __future__ import annotations

import asyncio
import sys
from pathlib import Path

import pytest
from mcp import ClientSession, StdioServerParameters

from a2c_smcp.computer.mcp_clients.base_client import BaseMCPClient
from a2c_smcp.computer.mcp_clients.stdio_client import StdioMCPClient

# ------------------------------
# 工具函数 / helpers
# ------------------------------


def _server_path(name: str) -> Path:
    return Path(__file__).resolve().parents[2] / "computer" / "mcp_servers" / name


# ------------------------------
# 覆盖 list_tools 分页 / tools pagination
# ------------------------------


@pytest.mark.asyncio
async def test_list_tools_pagination() -> None:
    """
    中文: 连接分页工具服务器，覆盖 list_tools 的 nextCursor 循环。
    英文: Connect paged tools server to cover list_tools nextCursor loop.
    """
    server_py = _server_path("tools_paged_stdio_server.py")
    assert server_py.exists()

    client = StdioMCPClient(StdioServerParameters(command=sys.executable, args=[str(server_py)]))
    await client.aconnect()
    await client._create_session_success_event.wait()

    tools = await client.list_tools()
    names = {t.name for t in tools}
    assert {"page1_tool", "page2_tool"}.issubset(names)

    await client.adisconnect()
    await client._async_session_closed_event.wait()


# ------------------------------
# 覆盖 list_windows 分页与过滤 / windows pagination + filtering
# ------------------------------


@pytest.mark.asyncio
async def test_list_windows_pagination_and_filtering() -> None:
    """
    中文: 连接分页+混合资源服务器，覆盖 list_resources 的分页与非 window 过滤。
    英文: Connect paged+mixed resources server to cover pagination and non-window filtering.
    """
    server_py = _server_path("resources_paged_mixed_stdio_server.py")
    assert server_py.exists()

    client = StdioMCPClient(StdioServerParameters(command=sys.executable, args=[str(server_py)]))
    await client.aconnect()
    await client._create_session_success_event.wait()

    # subscribe=True 才会进入逻辑分支 / must be subscribe=True
    assert client.initialize_result and client.initialize_result.capabilities.resources
    assert client.initialize_result.capabilities.resources.subscribe is True

    windows = await client.list_windows()
    # 应仅包含 window://，且包含来自两页的 p1/p2，但不包含 file:// / only window:// present
    uris = [str(r.uri) for r in windows]
    assert all(u.startswith("window://") for u in uris)
    assert any("/p1" in u for u in uris)
    assert any("/p2" in u for u in uris)

    await client.adisconnect()
    await client._async_session_closed_event.wait()


# ------------------------------
# 覆盖 list_windows 异常返回 [] / list_windows error -> []
# ------------------------------


@pytest.mark.asyncio
async def test_list_windows_error_returns_empty(monkeypatch) -> None:  # noqa: PT019
    """
    中文: 模拟 list_resources 抛出异常，覆盖 list_windows 的异常路径并返回 []。
    英文: Simulate list_resources raising to cover list_windows exception path returning [].
    """
    server_py = _server_path("resources_subscribe_stdio_server.py")
    assert server_py.exists()

    client = StdioMCPClient(StdioServerParameters(command=sys.executable, args=[str(server_py)]))
    await client.aconnect()
    await client._create_session_success_event.wait()

    # monkeypatch ClientSession.list_resources 以触发异常 / raise from list_resources
    asession = await client.async_session

    async def boom(*args, **kwargs):  # noqa: ANN001, ANN002
        raise RuntimeError("boom")

    monkeypatch.setattr(asession, "list_resources", boom, raising=True)

    windows = await client.list_windows()
    assert windows == []

    await client.adisconnect()
    await client._async_session_closed_event.wait()


# ------------------------------
# 覆盖 get_window_detail 成功/异常 / get_window_detail ok/error
# ------------------------------


@pytest.mark.asyncio
async def test_get_window_detail_ok_and_error() -> None:
    """
    中文: 成功读取与异常读取。成功用字符串 URI 调用以覆盖 AnyUrl(resource) 分支；异常使用资源对象以覆盖 except 内的 resource.uri。
    英文: Success via str URI to cover AnyUrl(resource); error via Resource object so except can access resource.uri.
    """
    # OK: 使用分页混合服务器读取字符串 URI / read from str URI
    paged_py = _server_path("resources_paged_mixed_stdio_server.py")
    client_ok = StdioMCPClient(StdioServerParameters(command=sys.executable, args=[str(paged_py)]))
    await client_ok.aconnect()
    await client_ok._create_session_success_event.wait()

    windows = await client_ok.list_windows()
    assert windows
    uri_str = str(windows[0].uri)
    detail = await client_ok.get_window_detail(uri_str)
    # 返回应为 ReadResourceResult，包含 contents / should contain contents
    assert getattr(detail, "contents", None) is not None

    await client_ok.adisconnect()
    await client_ok._async_session_closed_event.wait()

    # ERROR: 使用抛出 read_resource 的服务器 / server raising on read_resource
    err_py = _server_path("resources_read_error_stdio_server.py")
    client_err = StdioMCPClient(StdioServerParameters(command=sys.executable, args=[str(err_py)]))
    await client_err.aconnect()
    await client_err._create_session_success_event.wait()

    windows2 = await client_err.list_windows()
    assert not windows2, "因为resources_read_error_stdio_server.py没有实现 subscribe 能力，因此直接返回空"


# ------------------------------
# 覆盖 _keep_alive_task 异常路径 / _keep_alive_task error path
# ------------------------------


class FailingClient(BaseMCPClient):
    """
    中文: 自定义客户端：创建会话阶段抛出异常，从而触发 _create_session_failure_event 与 aerror()。
    英文: Custom client that raises during _create_async_session to trigger failure path and aerror().
    """

    async def _create_async_session(self) -> ClientSession:  # type: ignore[override]
        raise RuntimeError("create session failed")


# ------------------------------
# 覆盖 _close_task 等待子任务报错分支 / _close_task generic exception path
# ------------------------------


@pytest.mark.asyncio
async def test_close_task_handles_non_cancel_error():
    """
    中文: 构造一个在取消后抛出非 CancelledError 的任务，覆盖 _close_task 的异常捕获分支。
    英文: Build a task that raises non-CancelledError on cancellation to cover _close_task error branch.
    """

    async def evil_task():
        try:
            await asyncio.Event().wait()
        except asyncio.CancelledError as e:
            # 在取消时抛出其他异常 / raise non-cancel after cancel
            raise RuntimeError("boom-after-cancel") from e

    # 生成一个假的客户端以便复用 _close_task / reuse _close_task
    class Dummy(BaseMCPClient):
        async def _create_async_session(self) -> ClientSession:  # type: ignore[override]
            raise NotImplementedError

    dummy = Dummy(params=StdioServerParameters(command=sys.executable, args=["-c", "print('x')"]))
    dummy._session_keep_alive_task = asyncio.create_task(evil_task())

    # 执行关闭逻辑，若分支覆盖成功则不会抛出异常 / close should swallow non-cancel errors and log
    await dummy._close_task()



"""
* 文件名: test_sync_client
* 作者: JQQ
* 创建日期: 2025/9/30
* 最后修改日期: 2025/9/30
* 版权: 2023 JQQ. All rights reserved.
* 依赖: pytest, unittest.mock
* 描述: 同步Agent客户端测试用例 / Synchronous Agent client test cases
"""

import uuid
from unittest.mock import MagicMock, patch

import pytest
from mcp.types import CallToolResult

from a2c_smcp.agent.auth import DefaultAgentAuthProvider
from a2c_smcp.agent.sync_client import SMCPAgentClient
from a2c_smcp.smcp import (
    CANCEL_TOOL_CALL_EVENT,
    SMCP_NAMESPACE,
    EnterOfficeNotification,
    GetToolsRet,
    LeaveOfficeNotification,
    SMCPTool,
    UpdateMCPConfigNotification,
)


class MockEventHandler:
    """模拟事件处理器 / Mock event handler"""

    def __init__(self) -> None:
        self.enter_office_calls: list[EnterOfficeNotification] = []
        self.leave_office_calls: list[LeaveOfficeNotification] = []
        self.update_config_calls: list[UpdateMCPConfigNotification] = []
        self.tools_received_calls: list[tuple[str, list[SMCPTool]]] = []
        # 记录传入的client实例 / Record passed client instances
        self.enter_office_clients: list[SMCPAgentClient] = []
        self.leave_office_clients: list[SMCPAgentClient] = []
        self.update_config_clients: list[SMCPAgentClient] = []
        self.tools_received_clients: list[SMCPAgentClient] = []

    def on_computer_enter_office(self, data: EnterOfficeNotification, sio: SMCPAgentClient) -> None:
        self.enter_office_calls.append(data)
        self.enter_office_clients.append(sio)

    def on_computer_leave_office(self, data: LeaveOfficeNotification, sio: SMCPAgentClient) -> None:
        self.leave_office_calls.append(data)
        self.leave_office_clients.append(sio)

    def on_computer_update_config(self, data: UpdateMCPConfigNotification, sio: SMCPAgentClient) -> None:
        self.update_config_calls.append(data)
        self.update_config_clients.append(sio)

    def on_tools_received(self, computer: str, tools: list[SMCPTool], sio: SMCPAgentClient) -> None:
        self.tools_received_calls.append((computer, tools))
        self.tools_received_clients.append(sio)


class TestSMCPAgentClient:
    """测试同步SMCP Agent客户端 / Test synchronous SMCP Agent client"""

    @pytest.fixture
    def auth_provider(self) -> DefaultAgentAuthProvider:
        """创建认证提供者 / Create authentication provider"""
        return DefaultAgentAuthProvider(
            agent_id="test_agent",
            office_id="test_office",
            api_key="test_key",
        )

    @pytest.fixture
    def event_handler(self) -> MockEventHandler:
        """创建事件处理器 / Create event handler"""
        return MockEventHandler()

    @pytest.fixture
    def client(self, auth_provider: DefaultAgentAuthProvider, event_handler: MockEventHandler) -> SMCPAgentClient:
        """创建客户端实例 / Create client instance"""
        return SMCPAgentClient(auth_provider=auth_provider, event_handler=event_handler)

    def test_init(self, client: SMCPAgentClient) -> None:
        """测试客户端初始化 / Test client initialization"""
        assert client.auth_provider is not None
        assert client.event_handler is not None

    def test_validate_emit_event_notify(self, client: SMCPAgentClient) -> None:
        """测试验证notify事件被拒绝 / Test validate notify events are rejected"""
        with pytest.raises(ValueError, match="AgentClient不允许使用notify"):
            client.emit("notify:test_event")

    def test_validate_emit_event_agent(self, client: SMCPAgentClient) -> None:
        """测试验证agent事件被拒绝 / Test validate agent events are rejected"""
        with pytest.raises(ValueError, match="AgentClient不允许发起agent"):
            client.emit("agent:test_event")

    def test_validate_emit_event_valid(self, client: SMCPAgentClient) -> None:
        """测试验证有效事件通过 / Test validate valid events pass"""
        with patch.object(client, "emit", wraps=client.emit) as mock_emit:
            # 模拟父类emit方法
            # Mock parent class emit method
            with patch("socketio.Client.emit"):
                client.emit("client:test_event")
                mock_emit.assert_called_once()

    @patch("socketio.Client.call")
    def test_emit_tool_call_success(self, mock_call: MagicMock, client: SMCPAgentClient) -> None:
        """测试成功的工具调用 / Test successful tool call"""
        # 模拟成功响应
        # Mock successful response
        mock_response = {
            "content": [{"text": "Success", "type": "text"}],
            "isError": False,
        }
        mock_call.return_value = mock_response

        result = client.emit_tool_call(
            computer="test_computer",
            tool_name="test_tool",
            params={"param1": "value1"},
            timeout=30,
        )

        assert isinstance(result, CallToolResult)
        assert not result.isError
        mock_call.assert_called_once()

    @patch("socketio.Client.call")
    @patch("socketio.Client.emit")
    def test_emit_tool_call_timeout(self, mock_emit: MagicMock, mock_call: MagicMock, client: SMCPAgentClient) -> None:
        """测试工具调用超时 / Test tool call timeout"""
        # 模拟超时异常
        # Mock timeout exception
        mock_call.side_effect = TimeoutError("Timeout")

        result = client.emit_tool_call(
            computer="test_computer",
            tool_name="test_tool",
            params={"param1": "value1"},
            timeout=30,
        )

        assert isinstance(result, CallToolResult)
        assert result.isError
        assert "超时" in result.content[0].text  # type: ignore

        # 验证发送了取消请求
        # Verify cancel request was sent
        mock_emit.assert_called_once()
        args, kwargs = mock_emit.call_args
        assert args[1] == CANCEL_TOOL_CALL_EVENT
        assert args[3] == SMCP_NAMESPACE  # namespace 是第三个位置参数

    @patch("socketio.Client.call")
    def test_get_tools_from_computer_success(self, mock_call: MagicMock, client: SMCPAgentClient) -> None:
        """测试成功获取工具列表 / Test successful get tools list"""
        # 模拟工具响应
        # Mock tools response
        req_id = uuid.uuid4().hex
        mock_response = {
            "tools": [
                {
                    "name": "test_tool",
                    "description": "Test tool",
                    "params_schema": {},
                    "return_schema": None,
                },
            ],
            "req_id": req_id,
        }

        with patch("uuid.uuid4") as mock_uuid:
            mock_uuid.return_value.hex = req_id
            mock_call.return_value = mock_response

            result = client.get_tools_from_computer("test_computer")

            assert isinstance(result, dict)
            assert "tools" in result
            assert len(result["tools"]) == 1
            assert result["req_id"] == req_id

    @patch("socketio.Client.call")
    def test_get_tools_from_computer_invalid_response(self, mock_call: MagicMock, client: SMCPAgentClient) -> None:
        """测试获取工具列表响应无效 / Test get tools list invalid response"""
        # 模拟无效响应
        # Mock invalid response
        mock_response = {
            "tools": [],
            "req_id": "wrong_id",
        }
        mock_call.return_value = mock_response

        with pytest.raises(ValueError, match="Invalid response"):
            client.get_tools_from_computer("test_computer")

    def test_handle_computer_enter_office(self, client: SMCPAgentClient, event_handler: MockEventHandler) -> None:
        """测试处理Computer加入办公室事件 / Test handle Computer enter office event"""
        data: EnterOfficeNotification = {
            "office_id": "test_office",
            "computer": "test_computer",
        }

        with patch.object(client, "get_tools_from_computer") as mock_get_tools:
            mock_get_tools.return_value = {"tools": [], "req_id": "test_req"}

            client._on_computer_enter_office(data)

            # 验证事件处理器被调用
            # Verify event handler was called
            assert len(event_handler.enter_office_calls) == 1
            assert event_handler.enter_office_calls[0] == data

            # 验证获取工具被调用
            # Verify get tools was called
            mock_get_tools.assert_called_once_with("test_computer")

    def test_handle_computer_leave_office(self, client: SMCPAgentClient, event_handler: MockEventHandler) -> None:
        """测试处理Computer离开办公室事件 / Test handle Computer leave office event"""
        data: LeaveOfficeNotification = {
            "office_id": "test_office",
            "computer": "test_computer",
        }

        client._on_computer_leave_office(data)

        # 验证事件处理器被调用
        # Verify event handler was called
        assert len(event_handler.leave_office_calls) == 1
        assert event_handler.leave_office_calls[0] == data

    def test_sio_param_passed_to_enter_office_handler(self, client: SMCPAgentClient, event_handler: MockEventHandler) -> None:
        """测试sio参数被正确传入enter_office处理器 / Test sio param is correctly passed to enter_office handler"""
        data: EnterOfficeNotification = {
            "office_id": "test_office",
            "computer": "test_computer",
        }

        with patch.object(client, "get_tools_from_computer") as mock_get_tools:
            mock_get_tools.return_value = {"tools": [], "req_id": "test_req"}

            client._on_computer_enter_office(data)

            # 验证client实例被传入 / Verify client instance was passed
            assert len(event_handler.enter_office_clients) == 1
            passed_client = event_handler.enter_office_clients[0]

            # 验证传入的是同一个client实例 / Verify it's the same client instance
            assert passed_client is client

            # 验证可以访问client的属性 / Verify can access client properties
            assert hasattr(passed_client, "auth_provider")
            assert passed_client.auth_provider is not None
            assert hasattr(passed_client, "event_handler")

    def test_sio_param_passed_to_leave_office_handler(self, client: SMCPAgentClient, event_handler: MockEventHandler) -> None:
        """测试sio参数被正确传入leave_office处理器 / Test sio param is correctly passed to leave_office handler"""
        data: LeaveOfficeNotification = {
            "office_id": "test_office",
            "computer": "test_computer",
        }

        client._on_computer_leave_office(data)

        # 验证client实例被传入 / Verify client instance was passed
        assert len(event_handler.leave_office_clients) == 1
        passed_client = event_handler.leave_office_clients[0]

        # 验证传入的是同一个client实例 / Verify it's the same client instance
        assert passed_client is client
        assert isinstance(passed_client, SMCPAgentClient)

    def test_sio_param_passed_to_update_config_handler(self, client: SMCPAgentClient, event_handler: MockEventHandler) -> None:
        """测试sio参数被正确传入update_config处理器 / Test sio param is correctly passed to update_config handler"""
        data: UpdateMCPConfigNotification = {
            "computer": "test_computer",
        }

        with patch.object(client, "get_tools_from_computer") as mock_get_tools:
            mock_get_tools.return_value = {"tools": [], "req_id": "test_req"}

            client._on_computer_update_config(data)

            # 验证client实例被传入 / Verify client instance was passed
            assert len(event_handler.update_config_clients) == 1
            passed_client = event_handler.update_config_clients[0]

            # 验证传入的是同一个client实例 / Verify it's the same client instance
            assert passed_client is client
            assert isinstance(passed_client, SMCPAgentClient)

    def test_sio_param_passed_to_tools_received_handler(self, client: SMCPAgentClient, event_handler: MockEventHandler) -> None:
        """测试sio参数被正确传入tools_received处理器 / Test sio param is correctly passed to tools_received handler"""
        tools = [
            SMCPTool(
                name="test_tool",
                description="Test tool",
                params_schema={},
                return_schema=None,
            ),
        ]
        response: GetToolsRet = {
            "tools": tools,
            "req_id": "test_req",
        }

        client.process_tools_response(response, "test_computer")

        # 验证client实例被传入 / Verify client instance was passed
        assert len(event_handler.tools_received_clients) == 1
        passed_client = event_handler.tools_received_clients[0]

        # 验证传入的是同一个client实例 / Verify it's the same client instance
        assert passed_client is client
        assert isinstance(passed_client, SMCPAgentClient)

    def test_handle_computer_update_config(self, client: SMCPAgentClient, event_handler: MockEventHandler) -> None:
        """测试处理Computer更新配置事件 / Test handle Computer update config event"""
        data: UpdateMCPConfigNotification = {
            "computer": "test_computer",
        }

        with patch.object(client, "get_tools_from_computer") as mock_get_tools:
            mock_get_tools.return_value = {"tools": [], "req_id": "test_req"}

            client._on_computer_update_config(data)

            # 验证事件处理器被调用
            # Verify event handler was called
            assert len(event_handler.update_config_calls) == 1
            assert event_handler.update_config_calls[0] == data

            # 验证获取工具被调用
            # Verify get tools was called
            mock_get_tools.assert_called_once_with("test_computer")

    def test_validate_office_data_valid(self, client: SMCPAgentClient) -> None:
        """测试验证有效的办公室数据 / Test validate valid office data"""
        data: EnterOfficeNotification = {
            "office_id": "test_office",
            "computer": "test_computer",
        }

        computer = client.validate_office_data(data)
        assert computer == "test_computer"

    def test_validate_office_data_invalid_office(self, client: SMCPAgentClient) -> None:
        """测试验证无效的办公室ID / Test validate invalid office ID"""
        data: EnterOfficeNotification = {
            "office_id": "wrong_office",
            "computer": "test_computer",
        }

        with pytest.raises(AssertionError, match="无效的办公室ID"):
            client.validate_office_data(data)

    def test_validate_office_data_invalid_computer(self, client: SMCPAgentClient) -> None:
        """测试验证无效的计算机ID / Test validate invalid computer ID"""
        data: EnterOfficeNotification = {
            "office_id": "test_office",
            "computer": "",
        }

        with pytest.raises(AssertionError, match="无效的计算机ID"):
            client.validate_office_data(data)

    @patch("socketio.Client.call")
    def test_get_computers_in_office_success(self, mock_call: MagicMock, client: SMCPAgentClient) -> None:
        """测试成功获取房间内的Computer列表 / Test successfully get computers list in office"""
        office_id = "test_office"
        req_id = f"list_computers_test_agent_{office_id}"

        # 模拟响应包含多个会话，包括computer和agent角色
        # Mock response with multiple sessions including computer and agent roles
        mock_response = {
            "req_id": req_id,
            "sessions": [
                {"sid": "comp1", "role": "computer", "computer_id": "computer-1"},
                {"sid": "comp2", "role": "computer", "computer_id": "computer-2"},
                {"sid": "agent1", "role": "agent", "agent_id": "agent-1"},
            ],
        }
        mock_call.return_value = mock_response

        computers = client.get_computers_in_office(office_id)

        # 验证只返回computer角色的会话 / Verify only computer role sessions are returned
        assert len(computers) == 2
        assert all(c["role"] == "computer" for c in computers)
        assert computers[0]["computer_id"] == "computer-1"
        assert computers[1]["computer_id"] == "computer-2"

        # 验证调用参数 / Verify call arguments
        mock_call.assert_called_once()

    @patch("socketio.Client.call")
    def test_get_computers_in_office_empty(self, mock_call: MagicMock, client: SMCPAgentClient) -> None:
        """测试房间内没有Computer时返回空列表 / Test return empty list when no computers in office"""
        office_id = "test_office"
        req_id = f"list_computers_test_agent_{office_id}"

        # 模拟响应只有agent角色，没有computer
        # Mock response with only agent role, no computer
        mock_response = {
            "req_id": req_id,
            "sessions": [
                {"sid": "agent1", "role": "agent", "agent_id": "agent-1"},
            ],
        }
        mock_call.return_value = mock_response

        computers = client.get_computers_in_office(office_id)

        # 验证返回空列表 / Verify empty list is returned
        assert len(computers) == 0

    @patch("socketio.Client.call")
    def test_get_computers_in_office_invalid_response(self, mock_call: MagicMock, client: SMCPAgentClient) -> None:
        """测试响应req_id不匹配时抛出异常 / Test raise exception when response req_id mismatches"""
        office_id = "test_office"

        # 模拟响应的req_id不匹配
        # Mock response with mismatched req_id
        mock_response = {
            "req_id": "wrong_req_id",
            "sessions": [],
        }
        mock_call.return_value = mock_response

        with pytest.raises(ValueError, match="Invalid response with mismatched req_id"):
            client.get_computers_in_office(office_id)

    @patch("socketio.Client.call")
    def test_get_computers_in_office_timeout(self, mock_call: MagicMock, client: SMCPAgentClient) -> None:
        """测试请求超时时抛出异常 / Test raise exception on timeout"""
        office_id = "test_office"
        mock_call.side_effect = TimeoutError("Request timeout")

        with pytest.raises(TimeoutError):
            client.get_computers_in_office(office_id, timeout=1)



# -*- coding: utf-8 -*-
# filename: param_server.py
# 一个带自定义参数读取能力的最简 MCP stdio 服务器
from __future__ import annotations

import os

from mcp.server.fastmcp import FastMCP

mcp = FastMCP("Param Server")


def _get_prefix() -> str:
    # 从环境变量读取一个前缀
    return os.environ.get("GREETING", "Hello")


@mcp.tool()
def config_value(name: str | None = None) -> str:
    """返回基于当前配置生成的问候语。Reads current env-based prefix."""
    prefix = _get_prefix()
    who = name or "World"
    return f"{prefix}, {who}!"


def main() -> None:
    mcp.run()


if __name__ == "__main__":
    main()



# -*- coding: utf-8 -*-
# filename: test_computer_desktop.py
# @Time    : 2025/10/02 16:22
# @Author  : JQQ
# @Email   : jqq1716@gmail.com
# @Software: PyCharm
"""
Computer.get_desktop 单元测试
Unit tests for Computer.get_desktop

说明 / Notes:
- 为了隔离不稳定的 organize 实现，本测试通过 monkeypatch 将
  `a2c_smcp.computer.computer.organize_desktop` 打桩，避免验证其内部策略；
- Since organize implementation is not stable yet, we patch
  `a2c_smcp.computer.computer.organize_desktop` to isolate get_desktop behavior.
"""

from types import SimpleNamespace
from unittest.mock import AsyncMock, MagicMock

import pytest
from mcp.types import ReadResourceResult, TextResourceContents

from a2c_smcp.computer.computer import Computer
from a2c_smcp.computer.mcp_clients.manager import MCPServerManager


@pytest.mark.asyncio
async def test_get_desktop_calls_manager_and_organize(monkeypatch):
    """
    中文: 确认 get_desktop 会调用 manager.list_windows，并将结果与 size、history 传入 organize_desktop。
    English: Ensure get_desktop invokes manager.list_windows and forwards windows/size/history to organize_desktop.
    """
    # 准备 mock manager / Prepare mock manager
    mock_manager = MagicMock(spec=MCPServerManager)
    # manager.get_windows_details 返回两条 window 记录（三元组：server, Resource-like, ReadResourceResult）
    res1 = SimpleNamespace(uri="window://mcp-a/w1")
    res2 = SimpleNamespace(uri="window://mcp-b/w2")
    d1 = ReadResourceResult(contents=[TextResourceContents(uri="window://mcp-a/w1", text="w1-content")])
    d2 = ReadResourceResult(contents=[TextResourceContents(uri="window://mcp-b/w2", text="w2-content")])
    mock_windows = [("mcp-a", res1, d1), ("mcp-b", res2, d2)]
    mock_manager.get_windows_details = AsyncMock(return_value=mock_windows)

    # 将构造函数替换为返回 mock_manager / Patch constructor to return mock manager
    monkeypatch.setattr("a2c_smcp.computer.computer.MCPServerManager", lambda *a, **kw: mock_manager)

    # 打桩 organize_desktop / Stub organize_desktop
    organized = ["window://mcp-a/w1", "window://mcp-b/w2"]
    mock_organize = AsyncMock(return_value=organized)
    monkeypatch.setattr("a2c_smcp.computer.computer.organize_desktop", mock_organize)

    # 实例化并启动 / Instantiate and boot
    computer = Computer(name="test")
    await computer.boot_up()

    # 调用 get_desktop / Call get_desktop
    size = 5
    uri = "window://mcp-a/w1"
    result = await computer.get_desktop(size=size, window_uri=uri)

    # 断言调用链 / Assertions
    mock_manager.get_windows_details.assert_awaited_once_with(uri)
    # history 通过实例方法读取，这里仅验证 organize 接口参数完整性（避免对 windows 作脆弱的严格比较）
    call = mock_organize.await_args
    assert call.kwargs["size"] == size
    assert isinstance(call.kwargs["history"], tuple)

    # 返回值为 organize 的产出 / Return value equals organize output
    assert result == organized


@pytest.mark.asyncio
async def test_get_desktop_when_manager_none_returns_empty():
    """
    中文: 当 mcp_manager 尚未初始化时，get_desktop 返回空列表。
    English: Return empty list when mcp_manager is not initialized.
    """
    computer = Computer(name="test")
    # 不调用 boot_up，保持 mcp_manager 为 None
    desktops = await computer.get_desktop()
    assert desktops == []



# -*- coding: utf-8 -*-
# filename: test_sync_client.py
# @Time    : 2025/9/30 22:55
# @Author  : JQQ
# @Email   : jqq1716@gmail.com
# @Software: PyCharm
"""
中文：SMCP 同步 Agent 客户端（SMCPAgentClient）的集成测试。
English: Integration tests for SMCPAgentClient (synchronous).
"""

import socket
import threading
import time
from typing import Literal
from unittest.mock import patch

import pytest
from mcp.types import CallToolResult
from socketio import Client, WSGIApp
from werkzeug.serving import make_server

from a2c_smcp.agent.auth import DefaultAgentAuthProvider
from a2c_smcp.agent.sync_client import SMCPAgentClient
from a2c_smcp.agent.types import AgentEventHandler
from a2c_smcp.smcp import (
    JOIN_OFFICE_EVENT,
    SMCP_NAMESPACE,
    EnterOfficeNotification,
    EnterOfficeReq,
    LeaveOfficeNotification,
    SMCPTool,
    UpdateMCPConfigNotification,
)
from a2c_smcp.utils.logger import logger
from tests.integration_tests.mock_sync_smcp_server import create_sync_smcp_socketio


@pytest.fixture
def sync_server_port() -> int:
    """动态分配可用端口 / Dynamically allocate available port"""
    with socket.socket() as s:
        s.bind(("127.0.0.1", 0))
        return s.getsockname()[1]


# 创建同步 SMCP 服务器
sio = create_sync_smcp_socketio()
sio.eio.start_service_task = False  # 禁用监控任务避免关闭时出错
wsgi_app = WSGIApp(sio, socketio_path="/socket.io")


class ServerThread(threading.Thread):
    """多线程服务器管理类"""

    def __init__(self, app, host: str, port: int) -> None:
        super().__init__(daemon=True)
        # threaded=True 非常重要，允许多线程处理客户端请求
        self.server = make_server(host, port, app, threaded=True)
        self.host = host
        self.port = port

    def run(self) -> None:
        logger.info(f"Starting Werkzeug server on {self.host}:{self.port}")
        self.server.serve_forever()

    def shutdown(self) -> None:
        logger.info("Shutting down Werkzeug server...")
        self.server.shutdown()


class _EH(AgentEventHandler):
    """
    中文：同步事件处理器，记录回调数据。
    English: Sync event handler recording callbacks.
    """

    def __init__(self) -> None:
        self.enter_events: list[EnterOfficeNotification] = []
        self.leave_events: list[LeaveOfficeNotification] = []
        self.update_events: list[UpdateMCPConfigNotification] = []
        self.tools_received: list[tuple[str, list[SMCPTool]]] = []

    def on_computer_enter_office(self, data: EnterOfficeNotification, sio: SMCPAgentClient) -> None:
        self.enter_events.append(data)

    def on_computer_leave_office(self, data: LeaveOfficeNotification, sio: SMCPAgentClient) -> None:
        self.leave_events.append(data)

    def on_computer_update_config(self, data: UpdateMCPConfigNotification, sio: SMCPAgentClient) -> None:
        logger.info(f"[DEBUG] UpdateMCPConfigNotification received: computer={data.get('computer')}")
        self.update_events.append(data)

    def on_tools_received(self, computer: str, tools: list[SMCPTool], sio: SMCPAgentClient) -> None:
        self.tools_received.append((computer, tools))


@pytest.fixture
def startup_and_shutdown_sync_smcp_server(sync_server_port: int):
    """启动和关闭同步 SMCP 服务器的测试夹具"""
    server_thread = ServerThread(wsgi_app, "localhost", sync_server_port)
    server_thread.start()
    logger.info("Starting SMCP server...")
    time.sleep(0.5)  # 等待服务器启动
    yield sync_server_port
    logger.info("Shutting down SMCP server...")
    server_thread.shutdown()


def _join_office(client: Client, role: Literal["computer", "agent"], office_id: str, name: str) -> None:
    """
    中文：通过 server:join_office 进入办公室（同步）。
    English: Join office via server:join_office (sync).
    """
    payload: EnterOfficeReq = {"role": role, "office_id": office_id, "name": name}
    ok, err = client.call(JOIN_OFFICE_EVENT, payload, namespace=SMCP_NAMESPACE)
    assert ok  # 只检查成功状态，忽略返回消息


def test_agent_receives_enter_and_tools_sync(startup_and_shutdown_sync_smcp_server):
    """
    中文：验证同步Agent收到Computer进入办公室事件，并自动拉取工具列表。
    English: Verify sync Agent receives enter event and auto fetches tools.
    """
    port = startup_and_shutdown_sync_smcp_server
    handler = _EH()
    office_id = "office-sync-1"
    auth = DefaultAgentAuthProvider(agent_id="robot-sync-1", office_id=office_id)
    agent = SMCPAgentClient(auth_provider=auth, event_handler=handler)

    # Agent 连接并加入办公室
    agent.connect_to_server(
        f"http://localhost:{port}",
        namespace=SMCP_NAMESPACE,
        socketio_path="/socket.io",
    )

    agent.join_office(office_id=office_id, agent_name="robot-sync-1", namespace=SMCP_NAMESPACE)

    # 使用线程启动Computer客户端
    def run_computer_client() -> None:
        logger.info("Starting Computer Client...")
        computer = Client()
        computer.connect(
            f"http://localhost:{port}",
            namespaces=[SMCP_NAMESPACE],
            socketio_path="/socket.io",
        )
        _join_office(computer, role="computer", office_id=office_id, name="comp-sync-01")

        # 等待断开信号
        disconnect_event.wait()
        computer.disconnect()

    disconnect_event = threading.Event()
    computer_thread = threading.Thread(target=run_computer_client)
    computer_thread.start()

    # 等待事件传播
    time.sleep(1.0)

    # 验证事件和工具接收
    assert handler.enter_events, "应收到进入办公室事件"
    assert handler.tools_received, "应收到工具列表"
    assert handler.tools_received[0][1][0]["name"] == "echo"

    # 清理
    disconnect_event.set()
    computer_thread.join()
    agent.disconnect()


def test_agent_tool_call_roundtrip_sync(startup_and_shutdown_sync_smcp_server):
    """
    中文：验证同步Agent发起工具调用，Computer返回CallToolResult。
    English: Verify sync Agent tool-call roundtrip.
    """
    port = startup_and_shutdown_sync_smcp_server
    handler = _EH()
    office_id = "office-sync-2"
    auth = DefaultAgentAuthProvider(agent_id="robot-sync-2", office_id=office_id)
    agent = SMCPAgentClient(auth_provider=auth, event_handler=handler)

    # Agent 连接并加入办公室
    agent.connect_to_server(
        f"http://localhost:{port}",
        namespace=SMCP_NAMESPACE,
        socketio_path="/socket.io",
    )

    agent.join_office(office_id=office_id, agent_name="robot-sync-2", namespace=SMCP_NAMESPACE)

    # 使用线程启动Computer客户端
    def run_computer_client() -> None:
        logger.info("Starting Computer Client...")
        computer = Client()
        computer.connect(
            f"http://localhost:{port}",
            namespaces=[SMCP_NAMESPACE],
            socketio_path="/socket.io",
        )
        _join_office(computer, role="computer", office_id=office_id, name="comp-sync-02")

        # 等待断开信号
        disconnect_event.wait()
        computer.disconnect()

    disconnect_event = threading.Event()
    computer_thread = threading.Thread(target=run_computer_client)
    computer_thread.start()

    # 等待Computer加入
    time.sleep(0.5)

    # 发起工具调用（使用Mock服务器返回的结果）
    res = agent.emit_tool_call(
        computer="mock_computer_id",  # 使用模拟的computer ID
        tool_name="echo",
        params={"text": "hi"},
        timeout=5,
    )

    assert isinstance(res, CallToolResult)
    assert not res.isError
    assert any(getattr(c, "text", None) == "mock tool result" for c in res.content)

    # 清理
    disconnect_event.set()
    computer_thread.join()
    agent.disconnect()


def test_agent_receives_update_config_sync(startup_and_shutdown_sync_smcp_server):
    """
    中文：验证当Computer发出更新配置，Agent收到并再次拉取工具列表（同步）。
    English: Verify sync Agent receives update-config and re-fetches tools.
    """
    port = startup_and_shutdown_sync_smcp_server
    handler = _EH()
    office_id = "office-sync-3"
    auth = DefaultAgentAuthProvider(agent_id="robot-sync-3", office_id=office_id)
    agent = SMCPAgentClient(auth_provider=auth, event_handler=handler)

    # Agent 连接并加入办公室
    agent.connect_to_server(
        f"http://localhost:{port}",
        namespace=SMCP_NAMESPACE,
        socketio_path="/socket.io",
    )

    agent.join_office(office_id=office_id, agent_name="robot-sync-3", namespace=SMCP_NAMESPACE)

    # 使用线程启动Computer客户端
    def run_computer_client() -> None:
        logger.info("Starting Computer Client...")
        computer = Client()
        computer.connect(
            f"http://localhost:{port}",
            namespaces=[SMCP_NAMESPACE],
            socketio_path="/socket.io",
        )
        computer_name = "comp-sync-03"
        logger.info(f"[DEBUG] Computer name: {computer_name}")
        _join_office(computer, role="computer", office_id=office_id, name=computer_name)

        # 等待初次工具拉取
        time.sleep(0.5)

        # 触发配置更新
        computer_sid = computer.namespaces[SMCP_NAMESPACE]
        logger.info(f"[DEBUG] Computer SID: {computer_sid}")
        ok, err = computer.call(
            "server:update_config",
            {"computer": computer_sid},
            namespace=SMCP_NAMESPACE,
            timeout=3,
        )
        assert ok and err is None

        # 等待断开信号
        disconnect_event.wait()
        computer.disconnect()

    disconnect_event = threading.Event()
    computer_thread = threading.Thread(target=run_computer_client)
    computer_thread.start()

    # 等待事件传播和配置更新
    time.sleep(2.0)

    # 验证收到了更新配置事件或工具重新拉取
    assert handler.update_events or len(handler.tools_received) >= 2

    # 清理
    disconnect_event.set()
    computer_thread.join()
    agent.disconnect()


def test_validate_emit_event_blocks_invalid():
    """
    中文：验证同步客户端对非法事件名进行阻断（来源于 BaseAgentClient.validate_emit_event）。
    English: Verify invalid events are blocked by validate_emit_event.
    """
    auth = DefaultAgentAuthProvider(agent_id="robot-x", office_id="office-x")
    agent = SMCPAgentClient(auth_provider=auth)

    with pytest.raises(ValueError):
        agent.emit("notify:abc", {})

    with pytest.raises(ValueError):
        agent.call("agent:do_something", {})


def test_get_computers_in_office_sync(startup_and_shutdown_sync_smcp_server):
    """
    中文：验证同步Agent可以获取房间内所有Computer的信息。
    English: Verify sync Agent can get all computers info in the office.
    """
    port = startup_and_shutdown_sync_smcp_server
    handler = _EH()
    office_id = "office-sync-4"
    auth = DefaultAgentAuthProvider(agent_id="robot-sync-4", office_id=office_id)
    agent = SMCPAgentClient(auth_provider=auth, event_handler=handler)

    # Agent连接并加入办公室 / Agent connects and joins office
    agent.connect_to_server(
        f"http://localhost:{port}",
        namespace=SMCP_NAMESPACE,
        socketio_path="/socket.io",
    )
    agent.join_office(office_id=office_id, agent_name="robot-sync-4", namespace=SMCP_NAMESPACE)

    # 使用线程启动两个Computer客户端 / Start two Computer clients in threads
    def run_computer_client(computer_name: str) -> None:
        logger.info(f"Starting Computer Client: {computer_name}")
        computer = Client()
        computer.connect(
            f"http://localhost:{port}",
            namespaces=[SMCP_NAMESPACE],
            socketio_path="/socket.io",
        )
        _join_office(computer, role="computer", office_id=office_id, name=computer_name)

        # 等待断开信号 / Wait for disconnect signal
        disconnect_event.wait()
        computer.disconnect()

    disconnect_event = threading.Event()
    computer_thread1 = threading.Thread(target=run_computer_client, args=("comp-sync-04-1",))
    computer_thread2 = threading.Thread(target=run_computer_client, args=("comp-sync-04-2",))
    computer_thread1.start()
    computer_thread2.start()

    # 等待所有客户端加入完成 / Wait for all clients to join
    time.sleep(1.0)

    # 调用get_computers_in_office获取Computer列表 / Call get_computers_in_office to get computers list
    computers = agent.get_computers_in_office(office_id)

    # 验证返回的Computer数量 / Verify number of computers returned
    assert len(computers) == 2, f"Expected 2 computers, got {len(computers)}"

    # 验证所有返回的会话都是computer角色 / Verify all returned sessions are computer role
    assert all(c["role"] == "computer" for c in computers), "All sessions should be computer role"

    # 验证Computer名称 / Verify computer names
    computer_names = {c["name"] for c in computers}
    assert "comp-sync-04-1" in computer_names, "comp-sync-04-1 should be in the list"
    assert "comp-sync-04-2" in computer_names, "comp-sync-04-2 should be in the list"

    # 从每个Computer获取工具列表 / Get tools list from each computer
    for computer in computers:
        computer_sid = computer["sid"]
        logger.info(f"Getting tools from computer: {computer['name']} (sid: {computer_sid})")

        # 调用get_tools_from_computer获取工具列表 / Call get_tools_from_computer to get tools list
        tools_ret = agent.get_tools_from_computer(computer_sid)

        # 验证返回结果 / Verify the result
        assert tools_ret is not None, f"Failed to get tools from computer {computer['name']}"
        assert tools_ret.get("tools"), "GetToolsRet should have 'tools' attribute"
        assert tools_ret.get("req_id"), "GetToolsRet should have 'req_id' attribute"
        logger.info(f"Successfully got {len(tools_ret['tools'])} tools from computer {computer['name']}")

    # 清理 / Cleanup
    disconnect_event.set()
    computer_thread1.join()
    computer_thread2.join()
    agent.disconnect()


def test_get_computers_in_office_empty_sync(startup_and_shutdown_sync_smcp_server):
    """
    中文：验证当房间内没有Computer时，同步Agent返回空列表。
    English: Verify sync Agent returns empty list when no computers in office.
    """
    port = startup_and_shutdown_sync_smcp_server
    handler = _EH()
    office_id = "office-sync-5"
    auth = DefaultAgentAuthProvider(agent_id="robot-sync-5", office_id=office_id)
    agent = SMCPAgentClient(auth_provider=auth, event_handler=handler)

    # Agent连接并加入办公室 / Agent connects and joins office
    agent.connect_to_server(
        f"http://localhost:{port}",
        namespace=SMCP_NAMESPACE,
        socketio_path="/socket.io",
    )
    agent.join_office(office_id=office_id, agent_name="robot-sync-5", namespace=SMCP_NAMESPACE)

    # 等待连接完成 / Wait for connection to complete
    time.sleep(0.5)

    # 调用get_computers_in_office，应该返回空列表 / Call get_computers_in_office, should return empty list
    computers = agent.get_computers_in_office(office_id)

    # 验证返回空列表 / Verify empty list is returned
    assert len(computers) == 0, f"Expected 0 computers, got {len(computers)}"

    # 清理 / Cleanup
    agent.disconnect()


def test_sync_agent_on_computer_enter_office_with_mock(startup_and_shutdown_sync_smcp_server):
    """
    中文：测试同步Agent Client能自动响应Computer Client的ENTER_OFFICE_EVENT。
    English: Test that sync Agent Client responds to Computer Client's ENTER_OFFICE_EVENT.
    """
    port = startup_and_shutdown_sync_smcp_server
    office_id = "test-office-mock"
    auth = DefaultAgentAuthProvider(agent_id="mock_robot_id", office_id=office_id)
    agent_client = SMCPAgentClient(auth_provider=auth)

    # Mock process_tools_response 方法来验证是否被调用
    with patch.object(agent_client, "process_tools_response") as mock_process_tools_response:
        # Agent 连接并加入办公室
        agent_client.connect_to_server(
            f"http://localhost:{port}",
            namespace=SMCP_NAMESPACE,
            socketio_path="/socket.io",
        )

        agent_client.call(
            JOIN_OFFICE_EVENT,
            {"office_id": office_id, "role": "agent", "name": "mock_robot_id"},
            namespace=SMCP_NAMESPACE,
        )

        # 使用线程启动Computer客户端
        def run_computer_client() -> None:
            logger.info("Starting Computer Client...")
            computer_id = "mock_computer_sid"
            computer_client = Client()
            computer_client.connect(
                f"http://localhost:{port}",
                namespaces=[SMCP_NAMESPACE],
                socketio_path="/socket.io",
            )

            enter_payload: EnterOfficeReq = {
                "office_id": office_id,
                "role": "computer",
                "name": computer_id,
            }
            logger.info("Sending ENTER_OFFICE_EVENT...")
            computer_client.call(JOIN_OFFICE_EVENT, enter_payload, namespace=SMCP_NAMESPACE)

            disconnect_event.wait()
            computer_client.disconnect()

        disconnect_event = threading.Event()
        computer_thread = threading.Thread(target=run_computer_client)
        computer_thread.start()

        # 等待Agent Client响应
        time.sleep(1.0)

        # 断言process_tools_response被触发（即Agent Client响应了事件）
        mock_process_tools_response.assert_called()

        # 断言process_tools_response参数正确
        mock_args, _ = mock_process_tools_response.call_args
        assert mock_args[0]["tools"][0]["name"] == "echo"
        assert mock_args[0]["tools"][0]["description"] == "echo text"

        # 清理
        disconnect_event.set()
        computer_thread.join()
        agent_client.disconnect()



# -*- coding: utf-8 -*-
# filename: test_server_e2e.py
# @Time    : 2025/10/05 14:12
# @Author  : JQQ
# @Email   : jqq1716@gmail.com
# @Software: PyCharm
"""
中文: Server 模块的端到端测试：基于真实 HTTP 服务与真实 socketio.Client，验证核心事件流。
English: End-to-end tests for the Server module: real HTTP service and real socketio.Client validating core event flows.
"""

from __future__ import annotations

import time
from typing import Any

import pytest

from a2c_smcp.smcp import (
    CANCEL_TOOL_CALL_EVENT,
    CANCEL_TOOL_CALL_NOTIFICATION,
    ENTER_OFFICE_NOTIFICATION,
    GET_DESKTOP_EVENT,
    GET_TOOLS_EVENT,
    JOIN_OFFICE_EVENT,
    LEAVE_OFFICE_EVENT,
    LEAVE_OFFICE_NOTIFICATION,
    SMCP_NAMESPACE,
    TOOL_CALL_EVENT,
    UPDATE_CONFIG_EVENT,
    UPDATE_CONFIG_NOTIFICATION,
    UPDATE_DESKTOP_EVENT,
    UPDATE_DESKTOP_NOTIFICATION,
    UPDATE_TOOL_LIST_EVENT,
    UPDATE_TOOL_LIST_NOTIFICATION,
)

pytestmark = pytest.mark.e2e


def _wait_until(cond, timeout: float = 2.0, step: float = 0.01) -> bool:
    """
    中文: 简易等待辅助函数，直到条件满足或超时。
    English: Simple wait helper until condition met or timeout.
    """
    end = time.time() + timeout
    while time.time() < end:
        if cond():
            return True
        time.sleep(step)
    return cond()


def _join_office(client, role: str, name: str, office_id: str) -> None:
    """中文: 加入房间 / English: Join office"""
    ok, err = client.call(
        JOIN_OFFICE_EVENT,
        {"role": role, "name": name, "office_id": office_id},
        namespace=SMCP_NAMESPACE,
        timeout=5,
    )
    if not (ok and err is None):
        raise RuntimeError(f"加入房间失败 / Failed to join office: ok={ok}, err={err}")


def _leave_office(client, office_id: str) -> None:
    """中文: 离开房间 / English: Leave office"""
    ok, err = client.call(
        LEAVE_OFFICE_EVENT,
        {"office_id": office_id},
        namespace=SMCP_NAMESPACE,
        timeout=5,
    )
    if not (ok and err is None):
        raise RuntimeError(f"离开房间失败 / Failed to leave office: ok={ok}, err={err}")


def test_server_end_to_end_flow(agent_client, computer_client):
    """
    中文:
      - 验证加入房间后的通知广播
      - 验证 Agent 获取 Computer 工具与桌面
      - 验证 Computer 更新通知广播
      - 验证 Agent 发起工具调用以及取消调用的通知
      - 验证离开房间的通知广播
    English:
      - Verify notifications on join
      - Verify Agent fetches Computer tools and desktop
      - Verify Computer update notifications
      - Verify Agent tool call and cancel notifications
      - Verify notifications on leave
    """

    room = "office-1"
    comp_name = "cmp-1"
    agent_name = "age-1"

    # 准备通知捕获容器 / Prepare notification captures
    agent_events: dict[str, list[Any]] = {
        ENTER_OFFICE_NOTIFICATION: [],
        UPDATE_CONFIG_NOTIFICATION: [],
        UPDATE_TOOL_LIST_NOTIFICATION: [],
        UPDATE_DESKTOP_NOTIFICATION: [],
        CANCEL_TOOL_CALL_NOTIFICATION: [],
        LEAVE_OFFICE_NOTIFICATION: [],
    }

    computer_events: dict[str, list[Any]] = {
        ENTER_OFFICE_NOTIFICATION: [],
        UPDATE_CONFIG_NOTIFICATION: [],
        UPDATE_TOOL_LIST_NOTIFICATION: [],
        UPDATE_DESKTOP_NOTIFICATION: [],
        CANCEL_TOOL_CALL_NOTIFICATION: [],
        LEAVE_OFFICE_NOTIFICATION: [],
    }

    # 在 Computer 客户端上实现请求处理：get_tools / get_desktop / tool_call
    # Implement request handlers on computer side (MUST register BEFORE joining)
    def _on_get_tools(data):
        # 返回最简工具列表 / minimal tool list
        return {
            "tools": [
                {
                    "name": "echo",
                    "description": "echo input",
                    "params_schema": {"type": "object"},
                    "return_schema": {"type": "object"},
                },
            ],
            "req_id": data.get("req_id", "r1"),
        }

    def _on_get_desktop(data):
        return {"desktops": ["win://1"], "req_id": data.get("req_id", "r2")}

    def _on_tool_call(data):
        # 回显实现 / echo implementation
        return {"ok": True, "echo": data}

    computer_client.on(GET_TOOLS_EVENT, _on_get_tools, namespace=SMCP_NAMESPACE)
    computer_client.on(GET_DESKTOP_EVENT, _on_get_desktop, namespace=SMCP_NAMESPACE)
    computer_client.on(TOOL_CALL_EVENT, _on_tool_call, namespace=SMCP_NAMESPACE)

    # 注册通知监听 / subscribe notifications
    for evt, store in agent_events.items():

        def make_handler(s, e=evt):
            def handler(data):
                s.append(data)

            return handler

        agent_client.on(evt, make_handler(store), namespace=SMCP_NAMESPACE)

    for evt, store in computer_events.items():

        def make_handler(s, e=evt):
            def handler(data):
                s.append(data)

            return handler

        computer_client.on(evt, make_handler(store), namespace=SMCP_NAMESPACE)

    # 等待确保事件处理器注册完成 / Wait to ensure event handlers are registered
    time.sleep(0.1)

    # 计算机先加入，Agent 后加入，Computer 应收到 Agent 加入的通知 / computer joins first, then agent joins
    _join_office(computer_client, role="computer", name=comp_name, office_id=room)

    # 等待一下以确保 session 建立 / small wait to ensure session saved
    time.sleep(0.1)

    _join_office(agent_client, role="agent", name=agent_name, office_id=room)

    # Computer 应该收到 Agent 加入的通知 / Computer should receive agent join notification
    assert _wait_until(lambda: len(computer_events[ENTER_OFFICE_NOTIFICATION]) >= 1)
    assert computer_events[ENTER_OFFICE_NOTIFICATION][0]["office_id"] == room
    assert "agent" in computer_events[ENTER_OFFICE_NOTIFICATION][0]

    # Agent 拉取工具列表 / Agent get tools
    agent_ret_tools = agent_client.call(
        GET_TOOLS_EVENT,
        {"computer": comp_name, "robot_id": agent_name, "req_id": "req-tools"},
        namespace=SMCP_NAMESPACE,
        timeout=2,
    )
    assert isinstance(agent_ret_tools, dict)
    assert agent_ret_tools.get("tools") and agent_ret_tools.get("req_id") == "req-tools"

    # Agent 拉取桌面 / Agent get desktop
    agent_ret_desktop = agent_client.call(
        GET_DESKTOP_EVENT,
        {"computer": comp_name, "robot_id": agent_name, "req_id": "req-desk"},
        namespace=SMCP_NAMESPACE,
        timeout=2,
    )
    assert agent_ret_desktop.get("desktops") == ["win://1"]
    assert agent_ret_desktop.get("req_id") == "req-desk"

    # Computer 广播配置更新、工具列表更新、桌面更新 / updates broadcasting
    computer_client.emit(UPDATE_CONFIG_EVENT, {"computer": comp_name}, namespace=SMCP_NAMESPACE)
    computer_client.emit(UPDATE_TOOL_LIST_EVENT, {"computer": comp_name}, namespace=SMCP_NAMESPACE)
    computer_client.emit(UPDATE_DESKTOP_EVENT, {"computer": comp_name}, namespace=SMCP_NAMESPACE)

    assert _wait_until(lambda: len(agent_events[UPDATE_CONFIG_NOTIFICATION]) >= 1)
    assert _wait_until(lambda: len(agent_events[UPDATE_TOOL_LIST_NOTIFICATION]) >= 1)
    assert _wait_until(lambda: len(agent_events[UPDATE_DESKTOP_NOTIFICATION]) >= 1)

    # Agent 发起工具调用 / Agent tool call
    tool_call_ret = agent_client.call(
        TOOL_CALL_EVENT,
        {
            "computer": comp_name,
            "tool_name": "echo",
            "params": {"x": 1},
            "timeout": 2,
            "agent": agent_name,
            "req_id": "req-tc",
        },
        namespace=SMCP_NAMESPACE,
        timeout=3,
    )
    # 验证返回结果 / Verify return result
    assert tool_call_ret.get("ok") is True
    assert tool_call_ret.get("echo", {}).get("tool_name") == "echo"

    # Agent 取消工具调用（广播通知给 Computer） / cancel tool call
    agent_client.emit(
        CANCEL_TOOL_CALL_EVENT,
        {"agent": agent_name, "req_id": "req-tc"},
        namespace=SMCP_NAMESPACE,
    )
    assert _wait_until(lambda: len(computer_events[CANCEL_TOOL_CALL_NOTIFICATION]) >= 1)

    # 离开办公室 / leave
    # Computer 先离开，Agent 应该收到通知 / Computer leaves first, Agent should receive notification
    _leave_office(computer_client, office_id=room)
    assert _wait_until(lambda: len(agent_events[LEAVE_OFFICE_NOTIFICATION]) >= 1)
    assert agent_events[LEAVE_OFFICE_NOTIFICATION][0]["office_id"] == room
    assert "computer" in agent_events[LEAVE_OFFICE_NOTIFICATION][0]

    # Agent 后离开，此时房间里已经没有其他人了 / Agent leaves last, no one else in the room
    _leave_office(agent_client, office_id=room)



# -*- coding: utf-8 -*-
# filename: test_smcp_namespace_desktop.py
# @Time    : 2025/10/05 13:28
# @Author  : A2C-SMCP
# @Email   : qa@a2c-smcp.local
# @Software: PyTest
"""
中文：SMCPNamespace 桌面协议相关的单元测试。
English: Unit tests for SMCPNamespace desktop-related behaviors.
"""

from unittest.mock import AsyncMock

import pytest

from a2c_smcp.server import SMCPNamespace
from a2c_smcp.smcp import GET_DESKTOP_EVENT, GetDeskTopReq


@pytest.mark.asyncio
async def test_on_client_get_desktop_and_update_broadcast(monkeypatch: pytest.MonkeyPatch):
    ns = SMCPNamespace(auth_provider=AsyncMock())
    ns.server = AsyncMock()

    agent_sid = "a2"
    agent_name = "an2"
    comp_sid = "c2"
    comp_name = "cn2"
    sess_agent = {"role": "agent", "office_id": "room2"}
    sess_comp = {"role": "computer", "office_id": "room2"}

    # get_session: 查询 computer_sid 与 agent_sid
    async def _get_session(sid):
        return sess_comp if sid == comp_sid else sess_agent

    ns.get_session = AsyncMock(side_effect=_get_session)
    ns._name_to_sid_map = {agent_name: agent_sid, comp_name: comp_sid}
    # 转发到 client:get_desktop
    ns.call = AsyncMock(return_value={"desktops": ["window://m"], "req_id": "rid"})

    req: GetDeskTopReq = {"computer": comp_name, "robot_id": agent_name, "req_id": "rid", "desktop_size": 1}
    ret = await ns.on_client_get_desktop(agent_sid, req)
    assert ret["desktops"] == ["window://m"]
    ns.call.assert_awaited_with(GET_DESKTOP_EVENT, req, to=comp_sid, namespace=ns.namespace)

    # 广播 update_desktop
    ns.get_session = AsyncMock(return_value=sess_comp)
    ns.emit = AsyncMock()
    await ns.on_server_update_desktop(comp_sid, {"computer": comp_name})
    ns.emit.assert_awaited()



# -*- coding: utf-8 -*-
# filename: test_main_integration.py
# 基于真实 stdio MCP Server 的 CLI 集成测试
from __future__ import annotations

import json
from collections.abc import Callable
from contextlib import contextmanager
from typing import Any

import pytest
from mcp import StdioServerParameters
from typer.testing import CliRunner

import a2c_smcp.computer.cli.main as cli_main
from a2c_smcp.computer.cli.main import _interactive_loop
from a2c_smcp.computer.computer import Computer


class FakePromptSession:
    def __init__(self, commands: list[str]) -> None:
        self._commands = commands

    async def prompt_async(self, *_: str, **__: Any) -> str:
        if not self._commands:
            raise EOFError
        return self._commands.pop(0)


@contextmanager
def no_patch_stdout():
    yield


@pytest.mark.asyncio
async def test_cli_with_real_stdio(stdio_params: StdioServerParameters) -> None:
    """
    集成测试：通过 CLI 交互完成以下流程（使用真实 stdio MCP server 参数）：
    1) 添加 server 配置（disabled=false）
    2) 启动该 server
    3) 列出工具与状态
    4) 停止该 server
    5) 退出
    期望：流程执行无异常。
    """
    server_cfg = {
        "name": "it-stdio",
        "type": "stdio",
        "disabled": False,
        "forbidden_tools": [],
        "tool_meta": {},
        "server_parameters": json.loads(stdio_params.model_dump_json()),
    }

    commands = [
        f"server add {json.dumps(server_cfg)}",
        "start it-stdio",
        "tools",
        "status",
        "stop it-stdio",
        "exit",
    ]

    # Patch interactive IO
    cli_main.PromptSession = lambda: FakePromptSession(commands)  # type: ignore
    cli_main.patch_stdout = lambda raw: no_patch_stdout()  # type: ignore

    comp = Computer(name="test", inputs=set(), mcp_servers=set(), auto_connect=False, auto_reconnect=False)

    await _interactive_loop(comp)


class FakeSMCPClient:
    def __init__(self, *args: Any, **kwargs: Any) -> None:  # noqa: D401
        self.connected = False
        self.connect_args: dict[str, Any] | None = None
        FakeSMCPClient.last = self  # type: ignore[attr-defined]

    async def connect(self, url: str, auth: dict[str, Any] | None = None, headers: dict[str, Any] | None = None) -> None:
        self.connected = True
        self.connect_args = {"url": url, "auth": auth, "headers": headers}


@pytest.mark.asyncio
async def test_cli_socket_connect_guided_inputs_without_real_network(monkeypatch: pytest.MonkeyPatch) -> None:
    """
    集成层面验证 CLI 的交互式引导输入 URL/Auth/Headers 的行为，但不依赖真实网络。
    """
    # Patch client to fake
    cli_main.SMCPComputerClient = FakeSMCPClient  # type: ignore

    commands = [
        "socket connect",
        "http://127.0.0.1:9000",
        "apikey:xyz",
        "app:demo,build:42",
        "exit",
    ]

    # Patch interactive IO
    cli_main.PromptSession = lambda: FakePromptSession(commands)  # type: ignore
    cli_main.patch_stdout = lambda raw: no_patch_stdout()  # type: ignore

    comp = Computer(name="test", inputs=set(), mcp_servers=set(), auto_connect=False, auto_reconnect=False)
    await _interactive_loop(comp)

    last: FakeSMCPClient = FakeSMCPClient.last  # type: ignore[assignment]
    assert last.connected is True
    assert last.connect_args == {
        "url": "http://127.0.0.1:9000",
        "auth": {"apikey": "xyz"},
        "headers": {"app": "demo", "build": "42"},
    }


# ------------------------------
# 测试 --computer-factory 集成路径（通过 Typer CLI）
# ------------------------------


class _DummyInteractive:
    called: bool = False
    last_comp: Any | None = None

    @classmethod
    async def coro(cls, comp: Any, init_client: Any | None = None) -> None:  # noqa: ARG003
        cls.called = True
        cls.last_comp = comp


class _FakeComputer:
    """轻量 Computer 替身，匹配构造参数与异步上下文协议。"""

    def __init__(
        self,
        name: str,
        inputs: set[Any] | None = None,
        mcp_servers: set[Any] | None = None,
        auto_connect: bool = True,
        auto_reconnect: bool = True,
        confirm_callback: Callable[[str, str, str, dict], bool] | None = None,
        input_resolver: Any | None = None,
    ) -> None:
        self.init_args = {
            "name": name,
            "inputs": inputs,
            "mcp_servers": mcp_servers,
            "auto_connect": auto_connect,
            "auto_reconnect": auto_reconnect,
            "confirm_callback": confirm_callback,
            "input_resolver": input_resolver,
        }

    async def __aenter__(self) -> _FakeComputer:
        return self

    async def __aexit__(self, exc_type, exc, tb) -> None:  # noqa: ANN001
        return None


@pytest.fixture(autouse=True)
def _reset_dummy_interactive() -> None:
    _DummyInteractive.called = False
    _DummyInteractive.last_comp = None


def test_cli_root_with_computer_factory(monkeypatch: pytest.MonkeyPatch) -> None:
    """根路径（无子命令）携带 --computer-factory 时应调用解析的工厂。"""
    runner = CliRunner()

    # 工厂: 返回 _FakeComputer，并计数
    calls: dict[str, Any] = {"count": 0}

    def factory(**kwargs: Any) -> _FakeComputer:
        calls["count"] += 1
        return _FakeComputer(**kwargs)

    monkeypatch.setattr(cli_main, "resolve_import_target", lambda s: factory, raising=True)
    monkeypatch.setattr(cli_main, "_interactive_loop", _DummyInteractive.coro, raising=True)

    result = runner.invoke(cli_main.app, ["--computer-factory", "pkg.mod:factory"])  # noqa: S603

    assert result.exit_code == 0
    assert calls["count"] == 1
    assert _DummyInteractive.called is True
    assert isinstance(_DummyInteractive.last_comp, _FakeComputer)


def test_cli_run_with_computer_factory_fallback(monkeypatch: pytest.MonkeyPatch) -> None:
    """当解析到的目标不可调用时，CLI 应回退到默认 Computer；为便于断言，替换为 _FakeComputer。"""
    runner = CliRunner()

    monkeypatch.setattr(cli_main, "resolve_import_target", lambda s: object(), raising=True)
    monkeypatch.setattr(cli_main, "Computer", _FakeComputer, raising=True)
    monkeypatch.setattr(cli_main, "_interactive_loop", _DummyInteractive.coro, raising=True)

    result = runner.invoke(
        cli_main.app,
        [
            "run",
            "--computer-factory",
            "x.y:bad",
            "--auto-connect",
            "--auto-reconnect",
        ],
    )

    assert result.exit_code == 0
    assert _DummyInteractive.called is True
    assert isinstance(_DummyInteractive.last_comp, _FakeComputer)
    assert _DummyInteractive.last_comp.init_args["auto_connect"] is True
    assert _DummyInteractive.last_comp.init_args["auto_reconnect"] is True



# -*- coding: utf-8 -*-
# filename: test_manager_windows.py
# @Time    : 2025/10/02 19:02
# @Author  : JQQ
# @Email   : jqq1716@gmail.com
# @Software: PyCharm
"""
中文: 集成测试 MCPServerManager.list_windows，使用 resources_stdio_server 与 resources_subscribe_stdio_server。
英文: Integration tests for MCPServerManager.list_windows using resources_stdio_server and resources_subscribe_stdio_server.
"""

from __future__ import annotations

import sys
from pathlib import Path

import anyio
import pytest
from mcp import StdioServerParameters, types

from a2c_smcp.computer.mcp_clients.manager import MCPServerManager
from a2c_smcp.computer.mcp_clients.model import StdioServerConfig


@pytest.mark.anyio
async def test_manager_list_windows_aggregates_only_subscribe_server() -> None:
    """
    中文: Manager 应仅聚合开启 resources.subscribe 的服务的窗口；非订阅服务应返回空。
    英文: Manager should aggregate windows only from servers with resources.subscribe enabled; non-subscribe returns empty.
    """
    base = Path(__file__).resolve().parents[2] / "computer" / "mcp_servers"
    sub_py = base / "resources_subscribe_stdio_server.py"
    nosub_py = base / "resources_stdio_server.py"
    assert sub_py.exists() and nosub_py.exists()

    sub_params = StdioServerParameters(command=sys.executable, args=[str(sub_py)])
    nosub_params = StdioServerParameters(command=sys.executable, args=[str(nosub_py)])

    manager = MCPServerManager(auto_connect=False)
    sub_cfg = StdioServerConfig(name="srv_sub", server_parameters=sub_params)
    nosub_cfg = StdioServerConfig(name="srv_nosub", server_parameters=nosub_params)

    await manager.ainitialize([sub_cfg, nosub_cfg])
    await manager.astart_all()

    try:
        results = await manager.list_windows()
        # 只应来自订阅服务 / Only from subscribe server
        assert all(srv == "srv_sub" for srv, _ in results)
        assert len(results) >= 1

        # 验证排序（dashboard priority=90 在 main priority=60 之前）
        uris = [str(res.uri) for _, res in results]
        assert any("/dashboard" in u for u in uris)
        assert any("/main" in u for u in uris)
        if len(uris) >= 2:
            assert "/dashboard" in uris[0]
            assert "/main" in uris[1]
    finally:
        await manager.astop_all()


@pytest.mark.anyio
async def test_manager_list_windows_triggers_resource_updated_notification() -> None:
    """
    中文: Manager 在调用 list_windows 时，客户端会订阅窗口资源；订阅版服务器会立刻发送 ResourceUpdated 通知，
          因此注入的 message_handler 应该接收到该通知。
    英文: When Manager.list_windows triggers subscriptions, the subscribe-capable server immediately sends
          ResourceUpdated notifications; the injected message_handler should receive them.
    """
    base = Path(__file__).resolve().parents[2] / "computer" / "mcp_servers"
    sub_py = base / "resources_subscribe_stdio_server.py"
    assert sub_py.exists()

    sub_params = StdioServerParameters(command=sys.executable, args=[str(sub_py)])

    received: list[types.ResourceUpdatedNotification] = []

    async def message_handler(message):
        # 仅记录资源更新通知 / record only ResourceUpdatedNotification
        if isinstance(message, types.ServerNotification) and isinstance(
            message.root,
            types.ResourceUpdatedNotification,
        ):
            received.append(message.root)

    manager = MCPServerManager(auto_connect=False, message_handler=message_handler)
    sub_cfg = StdioServerConfig(name="srv_sub", server_parameters=sub_params)

    await manager.ainitialize([sub_cfg])
    await manager.astart_all()

    try:
        # 触发订阅 / trigger subscriptions
        results = await manager.list_windows()
        assert results, "should have windows to subscribe"
        listed_uris = {str(res.uri) for _, res in results}

        # 等待通知到达（最多2秒）/ wait up to 2s for notifications
        for _ in range(20):
            if received:
                break
            await anyio.sleep(0.1)

        assert received, "expected at least one ResourceUpdatedNotification"
        # 校验通知中的 URI 合理（属于已订阅的窗口之一）
        assert any(str(n.params.uri) in listed_uris for n in received)
    finally:
        await manager.astop_all()


@pytest.mark.anyio
async def test_manager_list_windows_filter_by_uri() -> None:
    """
    中文: Manager.list_windows(window_uri=...) 仅返回 URI 完全匹配的窗口与其 server 名称。
    英文: Manager.list_windows(window_uri=...) returns only the exact matched window with its server name.
    """
    base = Path(__file__).resolve().parents[2] / "computer" / "mcp_servers"
    sub_py = base / "resources_subscribe_stdio_server.py"
    assert sub_py.exists()

    sub_params = StdioServerParameters(command=sys.executable, args=[str(sub_py)])

    manager = MCPServerManager(auto_connect=False)
    sub_cfg = StdioServerConfig(name="srv_sub", server_parameters=sub_params)

    await manager.ainitialize([sub_cfg])
    await manager.astart_all()

    try:
        # 先获取全部，找到一个 URI
        results_all = await manager.list_windows()
        assert results_all, "should have at least one window from subscribe server"
        target_uri = str(results_all[0][1].uri)

        # 过滤后仅返回匹配项且 server 名称为 srv_sub
        results_filtered = await manager.list_windows(window_uri=target_uri)
        assert len(results_filtered) == 1
        srv_name, res = results_filtered[0]
        assert srv_name == "srv_sub"
        assert str(res.uri) == target_uri
    finally:
        await manager.astop_all()
